// ***************************************************************************
//DESCRIPTION:
//		ColourString.cpp: implementation of the CColourString class.
//NOTES:
//		This class converts standard C++ code into HTML for display in
//		a web browser.
//
//USAGE:
//		CString m_sSource {Populate with text to convert}
//		....
//		CColourString colourConverter( m_sSource );
//		colourConverter.ConverToHtml();
//		CString sConvertedHtml = colourConverter.GetHtml();
//
//TODO:
//		Add support for continuation charactors
//		Syntax colour directives only if the preceeding word is appropiate.
//
//CREATED:
//		1-9-2000, 12:42:02 by john@mctainsh.com
//
//COPYRIGHT RUBISH:
//		Copyright 1999 (C) John McTainsh.
//		I am happy for any developer to use this code. I think it is pretty
//		cool and suites my purposes. Feel free to copy and alter whatever
//		you please. Keep me informed of any good things you do to it
//		and also what it has been used for (for my ego).
//
//		This is shareware and can be copied or distrabuted without my concent,
//		permision or knowledge. It would be uncool to use it without putting
//		my name in the credits (ego) but that is up to you.
//
//		This code is provide as is. If it dont work or dont do what it should
//		then dont come crying to me. It is full of bugs (undocumented features)
//		which you the user/developer must find to use it.
//
//		Remember, you to can become a Jedi master (VC++ developer) if you can
//		resist the temptations of the dark side of the force (VB).
//
//		PS
//			Comments regarding my spelling will NOT be kindly recieved.
//		PSS
//			All the best software is written by New Zealanders.
//
// ***************************************************************************

#include "stdafx.h"
#include "ColourString.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// ***************************************************************************
//DESCRIPTION:
//		Definition of keywords to be coloured and operators
//CREATED:
//		31-8-2000, 16:49:44 by john@mctainsh.com
// *************************************************************************** 
const TCHAR g_szKeywordsShouldBe[] = 
		_T(" main thread naked noreturn uuid dllexport dllimport ");
const TCHAR g_szKeywordsMicrosoft[] = 
		_T(" __asm __assume __multiple_inheritance __except __single_inheritance __based ")
		_T(" __virtual_inheritance __fastcall __finally __cdecl __try __declspec __inline ")
		_T(" __int8 __uuidof __int16 __int32 __int64 __leave __stdcall ");
const TCHAR g_szKeywordsStandard[] = 
		_T(" else struct enum switch auto template explicit this bool extern mutable ") 
		_T(" break false throw case namespace true catch new try float char for ")
		_T(" operator typedef class friend private typeid const goto protected typename ")
		_T(" const_cast if public union continue inline register unsigned reinterpret_cast using ")
		_T(" default int return delete short signed virtual sizeof void do static volatile ")
		_T(" double static_cast wmain dynamic_cast long while ");
const TCHAR g_szKeywordsDirective[] =
		_T(" #define #error #import #undef defined")
		_T(" #elif #if #include ")
		_T(" #else #ifdef #line ")
		_T(" #endif #ifndef #pragma ")
		_T(" defined ")								
		_T(" alloc_text comment init_seg1 optimize ")
		_T(" auto_inline component inline_depth pack ")
		_T(" bss_seg data_seg inline_recursion pointers_to_members1 ")
		_T(" check_stack function intrinsic setlocale ")
		_T(" code_seg hdrstop message vtordisp1 ")
		_T(" const_seg include_alias once warning ");

//Operators
const TCHAR g_szOperators[] =
		_T("-+*/^~%!&|<>=;,:[]{}()?");

//Html code inserts
#define FONT_COMMENT	_T("<font color=\"#006600\">")
#define FONT_KEYWORD	_T("<font color=\"#0088FF\">")
#define FONT_OPERATOR	_T("<font color=\"#FF0000\">")
#define FONT_NUMBER		_T("<font color=\"#6600FF\">")
#define FONT_QUOTE		_T("<font color=\"#FF00FF\">")
#define FONT_SETUP		_T("<font face=\"Courier\" size=\"2\">")
#define FONT_END		_T("</font>")
#define PRE_SETUP		_T("<pre>")
#define PRE_END			_T("</pre>")
#define NOBRAKE_SETUP	_T("<nobr>")
#define NOBRAKE_END		_T("</nobr>")
#define HTML_SPACE		_T("&nbsp;")

//Special charactors
#define SPACE			_T(' ')
#define TAB				_T('\t')
#define CARAGE_RETURN	_T('\r')
#define LINE_FEED		_T('\n')
#define SINGLE_QUOTE	_T('\'')
#define DOUBLE_QUOTE	_T('\"')
#define BACK_SLASH		_T('\\')
#define STAR			_T('*')
#define FOWARD_SLASH	_T('/')


// ***************************************************************************
//DESCRIPTION:
//		Setup inital values
//PARAMS:
//		sSourceCode		Source code to be converted.
//		bPreFormat		True if block is to be wrapped in the <pre> format code.
//		nTabInterval	Spacing between tabs.
//CREATED:
//		1-9-2000, 12:42:45 by john@mctainsh.com
// ***************************************************************************
CColourString::CColourString( CString sSourceCode, BOOL bPreFormat, int nTabInterval ) :
	m_sSource(	sSourceCode ),
	m_convState( eNormal ),
	m_bPreFormatting( bPreFormat ),
	m_nTabInterval(	nTabInterval )
{
	TRACE( _T("CColourString::CColourString(CString)\n") );
}

CColourString::~CColourString()
{
	TRACE( _T("CColourString::~CColourString()\n") );
}


// ***************************************************************************
//DESCRIPTION:
//		Return the build up Html. If convert has not been called
//		this will return nothing.
//CREATED:
//		1-9-2000, 13:53:15 by john@mctainsh.com
// ***************************************************************************
CString CColourString::GetHtml()
{
	TRACE( _T("CColourString::GetHtml()\n") );
	return m_sOut;
}

// ***************************************************************************
//DESCRIPTION:
//		Process the word that has been built.
//		This is usually called when a break is found in the word.
//CREATED:
//		1-9-2000, 13:25:41 by john@mctainsh.com
// ***************************************************************************
void CColourString::WriteWord()
{
	TRACE( _T("CColourString::WriteWord()\n") );
	//Skip if the word is empty
	if( m_sWord.GetLength() > 0 )
	{
		//Is it a number
		if( isdigit( m_sWord[0] ) )
		{
			m_sOut += FONT_NUMBER;
			m_sOut += m_sWord;
			m_sOut += FONT_END;
		}
		else
		{
			CString sSpWordSp = _T(' ') + m_sWord + _T(' ');
			//Is it a key word
			if( strstr( g_szKeywordsShouldBe,  sSpWordSp ) != NULL ||
				strstr( g_szKeywordsMicrosoft, sSpWordSp ) != NULL ||
				strstr( g_szKeywordsStandard,  sSpWordSp ) != NULL ||
				strstr( g_szKeywordsDirective, sSpWordSp ) != NULL )
			{
				m_sOut += FONT_KEYWORD;
				m_sOut += m_sWord;
				m_sOut += FONT_END;
			}
			else
			{
				m_sOut += m_sWord;
			}
		}
		m_sWord.Empty();
	}
}

// ***************************************************************************
//DESCRIPTION:
//		Write charactors into the quote until a new line OR the matching
//		Quote mark arrives
//PARAMS:
//		ch					Charactor to add
//		chLast				Previous charactor
//		chQuoteTerminator	Matching quote mark to look for
//CREATED:
//		1-9-2000, 14:49:22 by john@mctainsh.com
// ***************************************************************************
void CColourString::ProcessQuote( TCHAR ch, TCHAR chLast, TCHAR chQuoteTerminator )
{
	if( ch == CARAGE_RETURN )
	{
		m_sOut += FONT_END;
		NewLine();
		m_convState = eNormal;
	}
	else
	{
		m_sOut += ch;
		if( ch == chQuoteTerminator && chLast != BACK_SLASH )
		{
			m_convState = eNormal;
			m_sOut += FONT_END;					
		}
	}
}


// ***************************************************************************
//DESCRIPTION:
//		A Carrage return has been caught and to move to the next line
//CREATED:
//		1-9-2000, 14:21:45 by john@mctainsh.com
// ***************************************************************************
void CColourString::NewLine()
{
	TRACE( _T("CColourString::NewLine()\n") );
	if( !m_bPreFormatting )
		m_sOut += _T("<br>");
	m_sOut += CARAGE_RETURN;
	m_sOut += LINE_FEED;
	m_nColumn = 1;
}


// ***************************************************************************
//DESCRIPTION:
//		Main external routine called to convert the source code to html.
//		This is quite nasty in its layout and sequencing so pay special
//		attendion to the use of continue.
//CREATED:
//		1-9-2000, 13:28:14 by john@mctainsh.com
// ***************************************************************************
void CColourString::ConverToHtml()
{
	TRACE( _T("CColourString::ConverToHtml()\n") );

	//Reset data before starting
	m_nColumn = 1;					
	m_sWord.Empty();

	if( m_bPreFormatting )
		m_sOut = PRE_SETUP;
	else
		m_sOut = NOBRAKE_SETUP;

	int nCur = -1;									//Current Charactor being checked
	int	 nSourceLength = m_sSource.GetLength();		//Length of the imput source file

	//
	//Loop through all charactors here
	//
	while( nCur < ( nSourceLength - 1 ) )
	{
		//Get the Last, Current and Next charactors
		nCur++;										//Process next charactor
		m_nColumn++;								//Next Column
		TCHAR ch = m_sSource[nCur];					//Current Charactor
		TCHAR chNext = NULL;						//Next charactor
		TCHAR chLast = NULL;						//Last charactor
		if( ( nCur + 1 ) < nSourceLength )
			chNext = m_sSource[nCur+1];
		if( nCur > 0 )
			chLast = m_sSource[nCur-1];

		//Trace to help with debugging
		//TRACE( "%c", ch );

		//Ignore line feeds and process only Carrage returns
		if( ch == LINE_FEED )
		{
			m_nColumn--;							
			continue;
		}

		//Spaces are compressed so must convert to HTML Spaces
		if( ch == SPACE )
		{
			if( m_convState == eNormal )
				WriteWord();
			//If we are using <pre> than done convert spaces
			if( !m_bPreFormatting && 
				( chNext == SPACE || chNext == TAB ) )
				m_sOut += HTML_SPACE;
			else
				m_sOut += SPACE;
			continue;
		}
					
		//Apply Tabs at all times (Align to 1, 5, 9, 13, 17, 21, ....
		if( ch == TAB )
		{
			WriteWord();
			if( m_bPreFormatting )	
				m_sOut += SPACE;
			else 
				m_sOut += HTML_SPACE;
			while( ( m_nColumn % m_nTabInterval ) != 1 )
			{	
				if( m_bPreFormatting )	
					m_sOut += SPACE;
				else 
					m_sOut += HTML_SPACE;
				m_nColumn++;
			}
			continue;
		}


		//Line comments end at the end of the line
		if( m_convState == eLineComment )
		{
			if( ch == CARAGE_RETURN )
			{
				m_sOut += FONT_END;
				NewLine();
				m_convState = eNormal;
			}
			else
			{
				m_sOut += ch;
			}
			continue;
		}

		//Continous comment ends with a */
		if(  m_convState == eContinousComment )
		{
			if( ch == CARAGE_RETURN )
			{
				NewLine();
			}
			else
			{
				m_sOut += ch;
				if( ch == FOWARD_SLASH && chLast == STAR )
				{
					m_sOut += FONT_END;
					m_convState = eNormal;
				}
			}
			continue;
		}

		//We are now building a quotation so keep adding until the end of the quote
		if( m_convState == eDoubleQuote )
		{
			ProcessQuote( ch, chLast, DOUBLE_QUOTE );
			continue;
		}

		//We are now building a single so keep adding until the end of the quote
		if( m_convState == eSingleQuote )
		{
			ProcessQuote( ch, chLast, SINGLE_QUOTE );
			continue;
		}


		//Process under normal conditions
		switch( ch )
		{
			//New line
			case CARAGE_RETURN :
				WriteWord();
				NewLine();
				break;

			//Start of a quote
			case DOUBLE_QUOTE :
				WriteWord();
				m_convState = eDoubleQuote;
				m_sOut += FONT_QUOTE;
				m_sOut += ch;
				break;
			case SINGLE_QUOTE :
				WriteWord();
				m_convState = eSingleQuote;
				m_sOut += FONT_QUOTE;
				m_sOut += ch;
				break;

			//Start of a comment (Maybe?)
			case FOWARD_SLASH :
				WriteWord();
				switch( chNext )
				{
					case FOWARD_SLASH:
						m_convState = eLineComment;
						m_sOut += FONT_COMMENT;
						m_sOut += FOWARD_SLASH;
						m_sOut += FOWARD_SLASH;
						m_nColumn++;
						nCur++;
						break;
					case STAR:
						m_convState = eContinousComment;
						m_sOut += FONT_COMMENT;
						m_sOut += FOWARD_SLASH;
						m_sOut += STAR;
						m_nColumn++;
						nCur++;
						break;
					default:
						m_sOut += FONT_OPERATOR;
						m_sOut += FOWARD_SLASH;
						m_sOut += FONT_END;
				}
				break;

			//Every thing else is a operator or a word
			default :
				if( strchr( g_szOperators, ch ) != NULL )
				{
					WriteWord();
					m_sOut += FONT_OPERATOR;
					m_sOut += ch;
					m_sOut += FONT_END;
				}
				else
				{
					m_sWord += ch;
				}
		}
	}

	WriteWord();
	//Close the data with a font tag to return to the original document font.
	if( m_bPreFormatting )
		m_sOut += PRE_END;
	else
		m_sOut += PRE_END;
}

