


<!-- #include virtual="/Inc/NoteHeader.inc" -->  
<img src='/images/DownLoad.gif'>&nbsp;<a href="Callback.zip">Download 
Client and Server files - 22 Kb</a> 
<h1>Introduction.</h1>
<p>Calling back from a server into a client is a very useful tool to resolve problems 
such as catching remote events without suffering the overhead of high polling rates. 
In COM calling back is not simple process because we may be crossing process boundaries, 
or even machines and networks. One standard method to do this is be using connection 
points. These are easily generated using the wizard and are derived from the IDispatch 
interface. For a faster callback it is possible to pass an interface directly back 
to the client with is much easier to implement in C++. 
<p>In this tutorial we will create a server that can display a message box and call 
back into the client. We will also add a function in the server which when called 
will fire the callback into the client.. 
<ul>
<li><a href="#_CreatingTheServer">Creating the server</a></li>
<li><a href="#_CreatingCPPClient">Creating a C++ Client</a></li>
<li><a href="#_CreatingVBClient">Creating a Visual Basic Client</a></li>
</ul>
<h1>Creating the server.<a name="CreatingTheServer"></a></h1>
<p>These steps will create a COM server with a callback interface and two functions.</p>
<ol>
<li>Open visual studio and create a new ATL project called <code>DemoCallback</code>. 
Choose the defaults and Finish. </li>
<li>Select, Insert a new ATL object and choose the Simple Object. Use a short name 
of <code>CallbackServer</code> and under the attributes tab select the Custom interface 
radio button.</li>
<li>Build the project.</li>
<li>Open the <code>DemoCallback.idl</code> and just above <code>helpstring("ICallbackServer 
Interface"),</code>insert <code>oleautomation,</code>.</li>
<li>Just above this interface between the second <code>import</code> line and the<code>[</code> 
insert the following code. 
<pre>
     <font color="#006600">//</font>
    <font color="#006600">//Callback interface</font>
    <font color="#006600">//</font>
    <font color="#FF0000">[</font>
        object<font color="#FF0000">,</font>
        <font color="#0088FF">uuid</font><font color="#FF0000">(</font>D072EB42<font color="#FF0000">-</font>D7BD<font color="#FF0000">-</font><font color="#6600FF">46f2</font><font color="#FF0000">-</font><font color="#6600FF">85C0</font><font color="#FF0000">-</font>A7B285061859<font color="#FF0000">)</font><font color="#FF0000">,</font>
        oleautomation<font color="#FF0000">,</font>
        helpstring<font color="#FF0000">(</font><font color="#FF00FF">"ICallbackEntry Interface"</font><font color="#FF0000">)</font><font color="#FF0000">,</font>
        pointer_default<font color="#FF0000">(</font>unique<font color="#FF0000">)</font>
    <font color="#FF0000">]</font>
    interface ICallbackEntry <font color="#FF0000">:</font> IUnknown
    <font color="#FF0000">{</font>
        HRESULT JrmEvent1<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font><font color="#FF0000">;</font>
</pre>
</li>
<li>Close the idl file, compile and add two methods the <code>ICallbackServer</code> 
by right clicking on the interface in the class viewer. <code>SimulateCallback</code> 
and <code>JrmAdvise([in] ICallbackEntry* pEntry)</code>;. The interface should look 
like this with a different <code>uuid</code>; 
<pre>
    <font color="#006600">//</font>
    <font color="#006600">//Main server interface</font>
    <font color="#006600">//</font>
    <font color="#FF0000">[</font>
        object<font color="#FF0000">,</font>
        <font color="#0088FF">uuid</font><font color="#FF0000">(</font>F73886CA<font color="#FF0000">-</font><font color="#6600FF">1871</font><font color="#FF0000">-</font><font color="#6600FF">423D</font><font color="#FF0000">-</font><font color="#6600FF">9611</font><font color="#FF0000">-</font><font color="#6600FF">565095314BCF</font><font color="#FF0000">)</font><font color="#FF0000">,</font>
        oleautomation<font color="#FF0000">,</font>
        helpstring<font color="#FF0000">(</font><font color="#FF00FF">"ICallbackServer Interface"</font><font color="#FF0000">)</font><font color="#FF0000">,</font>
        pointer_default<font color="#FF0000">(</font>unique<font color="#FF0000">)</font>
    <font color="#FF0000">]</font>
    interface ICallbackServer <font color="#FF0000">:</font> IUnknown
    <font color="#FF0000">{</font>
        <font color="#FF0000">[</font>helpstring<font color="#FF0000">(</font><font color="#FF00FF">"method JrmAdvise"</font><font color="#FF0000">)</font><font color="#FF0000">]</font> 
            HRESULT JrmAdvise<font color="#FF0000">(</font><font color="#FF0000">[</font>in<font color="#FF0000">]</font>  ICallbackEntry<font color="#FF0000">*</font> pEntry<font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#FF0000">[</font>helpstring<font color="#FF0000">(</font><font color="#FF00FF">"method SimulateCallback"</font><font color="#FF0000">)</font><font color="#FF0000">]</font> 
            HRESULT SimulateCallback<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font><font color="#FF0000">;</font>
</pre>
</li>
<li>Now add an attribute to the <code>CCallbackServer</code> class of<code> ICallbackEntry* 
m_pCallback;</code> and set it to NULL in the constructor. Don't for get to free it 
in <code>FinalRelease</code>.</li>
<li>Modify the <code>CallbackServer.cpp</code> as follows;</li>
</ol>
<pre>
<font color="#006600">// ***************************************************************************</font>
<font color="#006600">//DESCRIPTION:</font>
<font color="#006600">//      Pass the interface pointer the client has exposed to be</font>
<font color="#006600">//      called back on. Call this once at startup.</font>
<font color="#006600">//CREATED:</font>
<font color="#006600">//      11-12-2000, 1:30:45 AM by john@mctainsh.com</font>
<font color="#006600">// ***************************************************************************</font>
STDMETHODIMP CCallbackServer<font color="#FF0000">:</font><font color="#FF0000">:</font>JrmAdvise<font color="#FF0000">(</font>ICallbackEntry <font color="#FF0000">*</font>pEntry<font color="#FF0000">)</font>
<font color="#FF0000">{</font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font> m_pCallback <font color="#FF0000">)</font>
        <font color="#0088FF">return</font> MAKE_HRESULT<font color="#FF0000">(</font> <font color="#6600FF">3</font><font color="#FF0000">,</font> FACILITY_RPC<font color="#FF0000">,</font> ERROR_REQ_NOT_ACCEP <font color="#FF0000">)</font><font color="#FF0000">;</font>
    m_pCallback <font color="#FF0000">=</font> pEntry<font color="#FF0000">;</font>
    m_pCallback<font color="#FF0000">-</font><font color="#FF0000">></font>AddRef<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#0088FF">return</font> S_OK<font color="#FF0000">;</font>
<font color="#FF0000">}</font>

<font color="#006600">// ***************************************************************************</font>
<font color="#006600">//DESCRIPTION:</font>
<font color="#006600">//      Calls back into the client using a client provided interface</font>
<font color="#006600">//CREATED:</font>
<font color="#006600">//      11-12-2000, 1:30:42 AM by john@mctainsh.com</font>
<font color="#006600">// ***************************************************************************</font>
STDMETHODIMP CCallbackServer<font color="#FF0000">:</font><font color="#FF0000">:</font>SimulateCallback<font color="#FF0000">(</font><font color="#FF0000">)</font>
<font color="#FF0000">{</font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font> m_pCallback <font color="#FF0000">=</font><font color="#FF0000">=</font> NULL <font color="#FF0000">)</font>
        <font color="#0088FF">return</font> MAKE_HRESULT<font color="#FF0000">(</font> <font color="#6600FF">3</font><font color="#FF0000">,</font> FACILITY_RPC<font color="#FF0000">,</font> ERROR_CONNECTION_UNAVAIL <font color="#FF0000">)</font><font color="#FF0000">;</font>
    m_pCallback<font color="#FF0000">-</font><font color="#FF0000">></font>JrmEvent1<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#0088FF">return</font> S_OK<font color="#FF0000">;</font>
<font color="#FF0000">}</font>
</pre>
<p> That's it! Rebuild all and check it performs build registration in the output 
window.</p>
<h1>Creating a C++ Client.<a name="CreatingCPPClient"></a></h1>
<p>Visual C++ is an excellent language to develop COM clients but does require a little 
more work and VB. As a trade of you are given much greater control such the option 
to control who, where and how the server is instanciated at run time. 
<ol>
<li>Create an MFC Dialog based application using the wizard.</li>
<li>Add the following to the top of the Dialogs header file. 
<pre>
<font color="#0088FF">#import</font> <font color="#FF00FF">"..\DemoCallback\debug\DemoCallback.dll"</font> named_guids raw_interfaces_only 
<font color="#0088FF">using</font> <font color="#0088FF">namespace</font> DEMOCALLBACKLib<font color="#FF0000">;</font>

<font color="#006600">/////////////////////////////////////////////////////////////////////////////</font>
<font color="#006600">// Connected interface called from Server</font>
<font color="#006600">//WARNING:  It is assumed this object is heap based. Always create it</font>
<font color="#006600">//          using new.</font>
<font color="#0088FF">class</font> CCallbackEntry <font color="#FF0000">:</font> <font color="#0088FF">public</font> ICallbackEntry
<font color="#FF0000">{</font>
<font color="#006600">//Operators</font>
<font color="#0088FF">public</font><font color="#FF0000">:</font>
    STDMETHODIMP JrmEvent1<font color="#FF0000">(</font><font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        MessageBox<font color="#FF0000">(</font> NULL<font color="#FF0000">,</font> <font color="#FF00FF">"Event Recieved at JrmEvent1()"</font><font color="#FF0000">,</font> <font color="#FF00FF">"Recieved from Server!"</font><font color="#FF0000">,</font> MB_OK <font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">return</font> S_OK<font color="#FF0000">;</font>
    <font color="#FF0000">}</font>

<font color="#006600">//</font>
<font color="#006600">//The following is general fluff that should be moved off into a template</font>
<font color="#006600">//</font>
<font color="#0088FF">public</font><font color="#FF0000">:</font>
   CCallbackEntry<font color="#FF0000">(</font><font color="#FF0000">)</font> <font color="#FF0000">:</font> m_nRefCount<font color="#FF0000">(</font><font color="#6600FF">0</font><font color="#FF0000">)</font> <font color="#FF0000">{</font><font color="#FF0000">}</font>
    STDMETHODIMP QueryInterface<font color="#FF0000">(</font>REFIID riid<font color="#FF0000">,</font> <font color="#0088FF">void</font><font color="#FF0000">*</font><font color="#FF0000">*</font> ppv<font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        <font color="#0088FF">if</font> <font color="#FF0000">(</font>riid <font color="#FF0000">=</font><font color="#FF0000">=</font> IID_IUnknown <font color="#FF0000">|</font><font color="#FF0000">|</font> riid <font color="#FF0000">=</font><font color="#FF0000">=</font> IID_ICallbackEntry<font color="#FF0000">)</font>
            <font color="#FF0000">*</font>ppv<font color="#FF0000">=</font><font color="#0088FF">this</font><font color="#FF0000">;</font>
        <font color="#0088FF">else</font>
            <font color="#0088FF">return</font> <font color="#FF0000">*</font>ppv<font color="#FF0000">=</font><font color="#6600FF">0</font><font color="#FF0000">,</font>E_NOINTERFACE<font color="#FF0000">;</font>
        AddRef<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">return</font> S_OK<font color="#FF0000">;</font>
    <font color="#FF0000">}</font>

    STDMETHODIMP_<font color="#FF0000">(</font>ULONG<font color="#FF0000">)</font> AddRef<font color="#FF0000">(</font><font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        <font color="#0088FF">return</font> <font color="#FF0000">+</font><font color="#FF0000">+</font>m_nRefCount<font color="#FF0000">;</font>
    <font color="#FF0000">}</font>

    STDMETHODIMP_<font color="#FF0000">(</font>ULONG<font color="#FF0000">)</font> Release<font color="#FF0000">(</font><font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        <font color="#0088FF">if</font> <font color="#FF0000">(</font><font color="#FF0000">!</font><font color="#FF0000">-</font><font color="#FF0000">-</font>m_nRefCount<font color="#FF0000">)</font>
        <font color="#FF0000">{</font>
            <font color="#0088FF">delete</font> <font color="#0088FF">this</font><font color="#FF0000">;</font>
            <font color="#0088FF">return</font> <font color="#6600FF">0</font><font color="#FF0000">;</font>
        <font color="#FF0000">}</font>
        <font color="#0088FF">return</font> m_nRefCount<font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
<font color="#006600">//Attributes </font>
<font color="#0088FF">private</font><font color="#FF0000">:</font>
    ULONG m_nRefCount<font color="#FF0000">;</font>
<font color="#FF0000">}</font><font color="#FF0000">;</font>
</pre>
This defines the object to be called back into.</li>
<li>In the class definition of the same file add attributes to point to the server 
object and our interface object to call back into. 
<pre>
<font color="#006600">//Attributes</font>
<font color="#0088FF">private</font><font color="#FF0000">:</font>
    ICallbackServer<font color="#FF0000">*</font> m_pServer<font color="#FF0000">;</font>                     <font color="#006600">//Connection to server</font>
    CCallbackEntry<font color="#FF0000">*</font>  m_pCallBack<font color="#FF0000">;</font>                   <font color="#006600">//Callback interface</font>
</pre>
</li>
<li>At the top of the Dialog class CPP file add the following helper.
<pre>
<font color="#006600">// NOTE: CheckHRESULT is a helper function that takes an HRESULT</font>
<font color="#006600">//       and a function name as parameters and displays an error</font>
<font color="#006600">//       message if the HRESULT's severity bit indicates an error</font>
<font color="#006600">//       has happened. This function never returns if an error</font>
<font color="#006600">//       has occurred.</font>
<font color="#0088FF">void</font> CheckHRESULT<font color="#FF0000">(</font>HRESULT hr<font color="#FF0000">,</font> <font color="#0088FF">const</font> <font color="#0088FF">char</font> <font color="#FF0000">*</font>pszCause<font color="#FF0000">)</font>
<font color="#FF0000">{</font>
    <font color="#0088FF">if</font> <font color="#FF0000">(</font>FAILED<font color="#FF0000">(</font>hr<font color="#FF0000">)</font><font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        <font color="#0088FF">char</font> sz<font color="#FF0000">[</font><font color="#6600FF">1024</font><font color="#FF0000">]</font><font color="#FF0000">;</font>
        <font color="#0088FF">char</font> szCaption<font color="#FF0000">[</font><font color="#6600FF">1024</font><font color="#FF0000">]</font><font color="#FF0000">;</font>
        strcpy<font color="#FF0000">(</font>szCaption<font color="#FF0000">,</font> pszCause<font color="#FF0000">)</font><font color="#FF0000">;</font>
        strcat<font color="#FF0000">(</font>szCaption<font color="#FF0000">,</font> <font color="#FF00FF">" failed!"</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">if</font> <font color="#FF0000">(</font><font color="#FF0000">!</font>FormatMessageA<font color="#FF0000">(</font>FORMAT_MESSAGE_FROM_SYSTEM<font color="#FF0000">,</font> <font color="#6600FF">0</font><font color="#FF0000">,</font> hr<font color="#FF0000">,</font> <font color="#6600FF">0</font><font color="#FF0000">,</font> sz<font color="#FF0000">,</font> <font color="#6600FF">1024</font><font color="#FF0000">,</font> <font color="#6600FF">0</font><font color="#FF0000">)</font><font color="#FF0000">)</font>
            strcpy<font color="#FF0000">(</font>sz<font color="#FF0000">,</font> <font color="#FF00FF">"Unknown error"</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
        MessageBox<font color="#FF0000">(</font><font color="#6600FF">0</font><font color="#FF0000">,</font> sz<font color="#FF0000">,</font> szCaption<font color="#FF0000">,</font> MB_SETFOREGROUND<font color="#FF0000">)</font><font color="#FF0000">;</font>
        DebugBreak<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
<font color="#FF0000">}</font>
</pre></li>
<li>In the CPP file of the Dialog, just before the return, initialise COM and create 
the server object. 
<pre>
    <font color="#006600">//</font>
    <font color="#006600">//Setup for COM communications</font>
    <font color="#006600">//</font>
    HRESULT hr <font color="#FF0000">=</font> CoInitialize<font color="#FF0000">(</font> NULL <font color="#FF0000">)</font><font color="#FF0000">;</font>
    CheckHRESULT<font color="#FF0000">(</font> hr<font color="#FF0000">,</font> <font color="#FF00FF">"CoInitialize"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>

    hr <font color="#FF0000">=</font> CoCreateInstance<font color="#FF0000">(</font> 
            CLSID_CallbackServer<font color="#FF0000">,</font> 
            NULL<font color="#FF0000">,</font> CLSCTX_ALL<font color="#FF0000">,</font> 
            <font color="#0088FF">__uuidof</font><font color="#FF0000">(</font> m_pServer <font color="#FF0000">)</font><font color="#FF0000">,</font> 
            <font color="#FF0000">(</font><font color="#0088FF">void</font><font color="#FF0000">*</font><font color="#FF0000">*</font><font color="#FF0000">)</font><font color="#FF0000">&</font>m_pServer <font color="#FF0000">)</font><font color="#FF0000">;</font>
    CheckHRESULT<font color="#FF0000">(</font> hr<font color="#FF0000">,</font> <font color="#FF00FF">"CoCreateInstance of CallbackServer"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>

    <font color="#006600">//</font>
    <font color="#006600">//Expose the callback interface</font>
    <font color="#006600">//</font>
    m_pCallBack <font color="#FF0000">=</font> <font color="#0088FF">new</font> CCallbackEntry<font color="#FF0000">;</font>
    m_pCallBack<font color="#FF0000">-</font><font color="#FF0000">></font>AddRef<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    hr <font color="#FF0000">=</font> m_pServer<font color="#FF0000">-</font><font color="#FF0000">></font>JrmAdvise<font color="#FF0000">(</font> m_pCallBack <font color="#FF0000">)</font><font color="#FF0000">;</font>
    CheckHRESULT<font color="#FF0000">(</font> hr<font color="#FF0000">,</font> <font color="#FF00FF">"JrmAdvise"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
</pre>
</li>
<li>Now finally call the server to stimulate the callback into our app with the OK 
button event handler.
<pre>
<font color="#006600">// ***************************************************************************</font>
<font color="#006600">//DESCRIPTION:</font>
<font color="#006600">//        When OK is pressed call a method on the control.</font>
<font color="#006600">//CREATED:</font>
<font color="#006600">//        20-12-2000, 8:48:56 PM by john@mctainsh.com</font>
<font color="#006600">// ***************************************************************************</font>
<font color="#0088FF">void</font> CDemoMFCDlg<font color="#FF0000">:</font><font color="#FF0000">:</font>OnOK<font color="#FF0000">(</font><font color="#FF0000">)</font> 
<font color="#FF0000">{</font>
    TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"CDemoMFCDlg::OnOK() \n"</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    HRESULT hr <font color="#FF0000">=</font> m_pServer<font color="#FF0000">-</font><font color="#FF0000">></font>SimulateCallback<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    CheckHRESULT<font color="#FF0000">(</font> hr<font color="#FF0000">,</font> <font color="#FF00FF">"SimulateCallback"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
<font color="#FF0000">}</font>
</pre>
</li>
</ol>
<h1>Creating a Visual Basic Client.<a name="CreatingVBClient"></a></h1>
<p>Using Visual Basic 6 is probably the easiest way to use the Callback server. I 
know very little about VB so use this code with caution. </p>
<ol>
<li>Start a new standard exe project in VB.</li>
<li>Add a large button <i>which can be pressed with our forehead</i>.</li>
<li>Under <u>P</u>roject-&gt;Refere<u>n</u>ces tick the <code>DemoCallback</code> 
box.</li>
<li>Open the source code by double clicking on the button.</li>
<li>Add the following to the file. 
<pre>
<font color="#0088FF">Dim</font> oCall <font color="#0088FF">as</font> DEMOCALLBACKLib.CallbackServer
<font color="#0088FF">Dim</font> cls1 <font color="#0088FF">As</font> Class1
<font color="#0088FF">Private</font> <font color="#0088FF">Sub</font> Form_Load<font color="#FF0000">(</font><font color="#FF0000">)</font> 
    <font color="#0088FF">Set</font> oCall <font color="#FF0000">=</font> <font color="#0088FF">New</font> CallbackServer 
    <font color="#0088FF">Set</font> cls1 <font color="#FF0000">=</font> <font color="#0088FF">New</font> Class1
    oCall.JrmAdvise cls1 
<font color="#0088FF">End</font> <font color="#0088FF">Sub</font> 
<font color="#0088FF">Private</font> <font color="#0088FF">Sub</font> Command1_Click<font color="#FF0000">(</font><font color="#FF0000">)</font>
    oCall.SimulateCallback
<font color="#0088FF">End</font> <font color="#0088FF">Sub</font></pre>
</li>
<li>Under <u>P</u>roject select Add <u>C</u>lass Module and accept the default.</li>
<li>Add <code>Implements ICallbackEntry</code> to the top of the file and select it 
from the left combo box.</li>
<li>Modify the code to look as follows <br>
<code>Private Sub ICallbackEntry_JrmEvent1() <br>
&nbsp;&nbsp;&nbsp;&nbsp;MsgBox "CallBack occured" <br>
End Sub</code></li>
<li>Close this file and run.</li>
</ol>
<p>The application should display a dialog and with a fat button. Pressing the button 
will call a method in the server that will use the Callback interface to run a function 
in the VB client.</p>
<!-- #include virtual="/Inc/NoteFooter.inc" --> 




