


<!-- #include virtual="/Inc/NoteHeader.inc" --> 


<h1>Description.</h1>
Smart pointers are a relatively simple method of accessing COM objects. They encapsulate much of the 
tricky code required to instantiate and use a COM object.
      
<h1>Where do the name come from?</h1>

<p>The name of the items in this example can be found in the <code>tlh</code> file
and class factory as follows;</p>

<table cellpadding=5 cellspacing=0 border=1>
	<tr bgcolor="Navy">
		<td><font color=white>Name here</font></td>
		<td><font color=white>Purpose</font></td>
		<td><font color=white>How to find</font></td></tr>
	<tr>
		<td><code>TrickyCom.exe</code></td>
		<td>Name of the file containing the object. May be a DLL or EXE.</td>
		<td>Path and name of file.</td></tr>
	<tr>
		<td><code>TRICKYCOMLib</code></td>
		<td>This is the Library name space</td>
		<td>The namespace is located in <code>.tlh</code> and begins with <code>namespace</code> and ends with <code>...Lib {</code></td></tr>
	<tr>
		<td><code>ICtrlFlasherPtr</code></td>
		<td>Smart pointer name</td>
		<td>Look in <code>.tlh</code> and add <code>Ptr</code> to the end of the first parameter in <code>_COM_SMARTPTR_TYPEDEF(ICtrlFlasher,</code></td></tr>
	<tr>
		<td><code>CtrlFlasher</code></td>
		<td>The coclass name</td>
		<td>Look in <code>.tlh</code> and use the name defines in <code>struct /* coclass */ CtrlFlasher;</code></td></tr>
	<tr>
		<td><code>Increment()</code> and <code>Count</code></td>
		<td>The methods and properties</td>
		<td>Look in <code>.tlh</code> for the <code>// Property data</code> and <code>// Wrapper methods for error-handling</code> sections</td></tr>
</table>
     
      <h1>Solution.</h1>
      
      
      <p>In <code>StdAfx.h</code> add.</p>
       
     
<pre>
<font color="#006600">//Reference Com object to create *.tlh and *.tli</font>
<font color="#0088FF">#import</font> <font color="#FF00FF">"C:\Temp\TrickyCom\Debug\TrickyCom.exe"</font>
<font color="#006600">//The namespace is located in *.tlh and ends in ...Lib {</font>
<font color="#0088FF">using</font> <font color="#0088FF">namespace</font> TRICKYCOMLib<font color="#FF0000">;</font>       
</pre>
 
      
<p>In <code>Gizzmo.cpp</code> in <code>BOOL CGizzmoApp::InitInstance();</code> or
any simular start-up function that is called only once.</p>

<pre>
    <font color="#006600">//Initialise the application for use with COM objects</font>
    HRESULT hr <font color="#FF0000">=</font> CoInitialize<font color="#FF0000">(</font> NULL <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font> FAILED<font color="#FF0000">(</font> hr <font color="#FF0000">)</font> <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        AfxMessageBox<font color="#FF0000">(</font> <font color="#FF00FF">"CoInitialize failed\n"</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">return</font> FALSE<font color="#FF0000">;</font>
    <font color="#FF0000">}</font>

    <font color="#006600">//TODO Call CoUninitialize(); in the Destructor or other location</font>
    <font color="#006600">//when all COM activity is completed.     </font>
</pre>

<p>In the header of the class place the object reference variable. <code>UseItHere.h</code></p>    
      
<pre>
    ICtrlFlashPtr m_pFlash<font color="#FF0000">;</font>                 <font color="#006600">//Smart pointer</font>
</pre>
      
<p>In class contructor initialise the variable. <code>UseItHere.cpp</code> Constructor.</p>    

<pre>
    m_pFlash <font color="#FF0000">=</font> NULL<font color="#FF0000">;</font>
</pre>

<p>In class destructor release the object. <code>UseItHere.cpp</code> Destructor.</p>    

<pre>
    m_pFlash.Release<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    m_pFlash <font color="#FF0000">=</font> NULL<font color="#FF0000">;</font>
</pre>

<p>In an initialisation routine such as <code>InitDialog</code> instantiate the object.
<code>UseItHere.cpp</code></p>    

<pre>
    <font color="#006600">//Instantiate the smart pointer</font>
    hr <font color="#FF0000">=</font> m_pFlash.CreateInstance<font color="#FF0000">(</font> <font color="#0088FF">__uuidof</font><font color="#FF0000">(</font> CtrlFlash <font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>

    <font color="#0088FF">if</font><font color="#FF0000">(</font> FAILED<font color="#FF0000">(</font> hr <font color="#FF0000">)</font> <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"ERROR : Instanciate FLASH FAILED %x!!!\n"</font><font color="#FF0000">)</font><font color="#FF0000">,</font> hr <font color="#FF0000">)</font><font color="#FF0000">;</font>
        AfxMessageBox<font color="#FF0000">(</font> <font color="#FF00FF">"Flash load failed\n"</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">return</font> FALSE<font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
</pre>

<p>Place this code where you want to access the object. ie In a button handler.</p>    

<pre>
    <font color="#0088FF">try</font>
    <font color="#FF0000">{</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"Inital count = %d\n"</font><font color="#FF0000">)</font><font color="#FF0000">,</font> pFlash<font color="#FF0000">-</font><font color="#FF0000">></font>nBeerCount <font color="#FF0000">)</font><font color="#FF0000">;</font>
        pFlash<font color="#FF0000">-</font><font color="#FF0000">></font>MoreBeer<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
    <font color="#0088FF">catch</font><font color="#FF0000">(</font> _com_error e <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        <font color="#006600">//Catch and display errors</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"Com Error %x, %s\n"</font><font color="#FF0000">)</font><font color="#FF0000">,</font> e.Error<font color="#FF0000">,</font> e.ErrorMessage<font color="#FF0000">(</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
</pre>      
            
<!-- #include virtual="/Inc/NoteFooter.inc" --> 




