<!-- #include virtual="/Inc/NoteHeader.inc" --> 

<h1>Description.</h1>
Use this method to use a COM object with the same process, or a remove process. 
This method does not require smart pointers so it is up to the 
developer to do the right thing when done with the object 
so it will be released.

<h1>Where do the names come from?</h1>

<p>The name of the items in this example can be found in the <code>tlh</code> file
and class factory as follows;</p>

<table cellpadding=5 cellspacing=0 border=1>
	<tr bgcolor="Navy">
		<td><font color=white>Name here</font></td>
		<td><font color=white>Purpose</font></td>
		<td><font color=white>How to find</font></td></tr>
	<tr>
		<td><code>TrickyCom.dll</code></td>
		<td>Name of the file containing the object. May be a DLL or EXE.</td>
		<td>Path and name of file.</td></tr>
	<tr>
		<td><code>CLSID_CtrlFlasher</code></td>
		<td>Class ID for the object.</td>
		<td>Located at the end of <code>.tlh</code>. Only created if the import directive includes named_guids</td></tr>
	<tr>
		<td><code>TRICKYCOMLib</code></td>
		<td>This is the Library name space</td>
		<td>The namespace is located in <code>.tlh</code> and begins with <code>namespace</code> and ends with <code>...Lib {</code></td></tr>
	<tr>
		<td><code>ICtrlFlasherPtr</code></td>
		<td>Smart pointer name</td>
		<td>Look in <code>.tlh</code> and add <code>Ptr</code> to the end of the first parameter in <code>_COM_SMARTPTR_TYPEDEF(ICtrlFlasher,</code></td></tr>
	<tr>
		<td><code>CtrlFlasher</code></td>
		<td>The coclass name</td>
		<td>Look in <code>.tlh</code> and use the name defines in <code>struct /* coclass */ CtrlFlasher;</code></td></tr>
	<tr>
		<td><code>Increment()</code> and <code>Count</code></td>
		<td>The methods and properties</td>
		<td>Look in <code>.tlh</code> for the <code>// Property data</code> and <code>// Wrapper methods for error-handling</code> sections</td></tr>
</table>

<h1>Solution using the object throughout a class.</h1>

<p>At the top of the file or in <code>StdAfx.h</code> add. Note 
the <code>named_guids</code> flag is very important.</p>
 
<pre>
<font color="#0088FF">#import</font> <font color="#FF00FF">"..\Debug\TrickyCom.exe"</font> named_guids
<font color="#0088FF">using</font> <font color="#0088FF">namespace</font> TRICKYCOMLib<font color="#FF0000">;</font>
</pre>
 
<p>In <code>Gizzmo.cpp</code> in <code>BOOL CGizzmoApp::InitInstance();</code> or
any simular start-up function that is called only once.</p>
 

<pre>
    <font color="#006600">//Initialise the application for use with COM objects</font>
    HRESULT hr <font color="#FF0000">=</font> CoInitialize<font color="#FF0000">(</font> NULL <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font> FAILED<font color="#FF0000">(</font> hr <font color="#FF0000">)</font> <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        AfxMessageBox<font color="#FF0000">(</font> <font color="#FF00FF">"CoInitialize failed\n"</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">return</font> FALSE<font color="#FF0000">;</font>
    <font color="#FF0000">}</font>

    <font color="#006600">//TODO Call CoUninitialize(); in the Destructor or other location</font>
    <font color="#006600">//when all COM activity is completed.</font>
</pre>
      
  
<p>In the header of the class place the object reference variable. <code>UseItHere.h</code></p>    

<pre>
    ICtrlFlash<font color="#FF0000">*</font> m_pFlash<font color="#FF0000">;</font>                 <font color="#006600">//Interface pointer</font>
</pre>	    

	<p>In class contructor initialise the variable. <code>UseItHere.cpp</code> Constructor.</p>    

<pre>
    m_pFlash <font color="#FF0000">=</font> NULL<font color="#FF0000">;</font>
</pre>

	<p>In class destructor release the object. <code>UseItHere.cpp</code> Destructor.</p>    

<pre>
    m_pFlash<font color="#FF0000">-</font><font color="#FF0000">></font>Release<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    m_pFlash <font color="#FF0000">=</font> NULL<font color="#FF0000">;</font>
</pre>


<p>In an initialisation routine such as <code>InitDialog</code> instantiate the object.
<code>UseItHere.cpp</code>. </p>    

<pre>
    <font color="#0088FF">try</font>
    <font color="#FF0000">{</font>
        HRESULT hr<font color="#FF0000">;</font>

        <font color="#006600">//</font>
        <font color="#006600">//Create the COM object</font>
        <font color="#006600">//</font>
        hr <font color="#FF0000">=</font> CoCreateInstance<font color="#FF0000">(</font> CLSID_CtrlFlash<font color="#FF0000">,</font> <font color="#6600FF">0</font><font color="#FF0000">,</font> CLSCTX_ALL<font color="#FF0000">,</font> IID_ICtrlFlash<font color="#FF0000">,</font> <font color="#FF0000">(</font><font color="#0088FF">void</font><font color="#FF0000">*</font><font color="#FF0000">*</font><font color="#FF0000">)</font><font color="#FF0000">&</font>m_pFlash <font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">if</font><font color="#FF0000">(</font> FAILED<font color="#FF0000">(</font>hr<font color="#FF0000">)</font> <font color="#FF0000">)</font>
            _com_issue_error<font color="#FF0000">(</font>hr<font color="#FF0000">)</font><font color="#FF0000">;</font>

    <font color="#FF0000">}</font>
    <font color="#0088FF">catch</font><font color="#FF0000">(</font> _com_error e <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"COM - Error %08x : %s\n"</font><font color="#FF0000">)</font><font color="#FF0000">,</font> e.Error<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">,</font> e.ErrorMessage<font color="#FF0000">(</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
</pre>

<p>Place this code where you want to access the object. ie In a button handler.</p>    
      
      
<pre>
    <font color="#0088FF">try</font>
    <font color="#FF0000">{</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"Inital count = %d\n"</font><font color="#FF0000">)</font><font color="#FF0000">,</font> pFlash<font color="#FF0000">-</font><font color="#FF0000">></font>nBeerCount <font color="#FF0000">)</font><font color="#FF0000">;</font>
        pFlash<font color="#FF0000">-</font><font color="#FF0000">></font>MoreBeer<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
    <font color="#0088FF">catch</font><font color="#FF0000">(</font> _com_error e <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        <font color="#006600">//Catch and display errors</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"Com Error %x, %s\n"</font><font color="#FF0000">)</font><font color="#FF0000">,</font> e.Error<font color="#FF0000">,</font> e.ErrorMessage<font color="#FF0000">(</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
</pre>      

<h1>All in one function</h1>

<p>
Here is another example doing every thing in the one function.
</p>

<pre>
    <font color="#006600">//Initialise the application for use with COM objects</font>
    HRESULT hr <font color="#FF0000">=</font> CoInitialize<font color="#FF0000">(</font> NULL <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font> FAILED<font color="#FF0000">(</font> hr <font color="#FF0000">)</font> <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        printf<font color="#FF0000">(</font> <font color="#FF00FF">"CoInitialize failed\n"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">return</font> FALSE<font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
    
    <font color="#006600">//Requires #import "..\..\Set01\Debug\Set01.dll" named_guids</font>
    IWhereAmI<font color="#FF0000">*</font> pWAI <font color="#FF0000">=</font> NULL<font color="#FF0000">;</font>
    <font color="#0088FF">try</font>
    <font color="#FF0000">{</font>      
        hr <font color="#FF0000">=</font> CoCreateInstance<font color="#FF0000">(</font> CLSID_WhereAmI<font color="#FF0000">,</font> <font color="#6600FF">0</font><font color="#FF0000">,</font> CLSCTX_ALL<font color="#FF0000">,</font> IID_IWhereAmI<font color="#FF0000">,</font> <font color="#FF0000">(</font><font color="#0088FF">void</font><font color="#FF0000">*</font><font color="#FF0000">*</font><font color="#FF0000">)</font><font color="#FF0000">&</font>pWAI <font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">if</font><font color="#FF0000">(</font> FAILED<font color="#FF0000">(</font> hr <font color="#FF0000">)</font> <font color="#FF0000">)</font>
            _com_issue_error<font color="#FF0000">(</font>hr<font color="#FF0000">)</font><font color="#FF0000">;</font>

        <font color="#006600">//Get the object version info</font>
        printf<font color="#FF0000">(</font> <font color="#FF00FF">"Inital count = %d\n"</font><font color="#FF0000">,</font> pWAI<font color="#FF0000">-</font><font color="#FF0000">></font>BuildNo <font color="#FF0000">)</font><font color="#FF0000">;</font>

        <font color="#006600">//Get the machine name</font>
        BSTR bstrMachineName<font color="#FF0000">;</font>
        hr <font color="#FF0000">=</font> pWAI<font color="#FF0000">-</font><font color="#FF0000">></font>GetMachine<font color="#FF0000">(</font> <font color="#FF0000">&</font>bstrMachineName <font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">if</font><font color="#FF0000">(</font> FAILED<font color="#FF0000">(</font> hr <font color="#FF0000">)</font> <font color="#FF0000">)</font>
            _com_issue_error<font color="#FF0000">(</font>hr<font color="#FF0000">)</font><font color="#FF0000">;</font>
        wprintf<font color="#FF0000">(</font> L<font color="#FF00FF">"Where is it  = %s\n"</font><font color="#FF0000">,</font> bstrMachineName <font color="#FF0000">)</font><font color="#FF0000">;</font>
        SysFreeString<font color="#FF0000">(</font> bstrMachineName <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
    <font color="#0088FF">catch</font><font color="#FF0000">(</font> _com_error e <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        <font color="#006600">//Catch and display errors</font>
        printf<font color="#FF0000">(</font> <font color="#FF00FF">"Com Error %x, %s\n"</font><font color="#FF0000">,</font> e.Error<font color="#FF0000">,</font> e.ErrorMessage<font color="#FF0000">(</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>           

    <font color="#006600">//Free up when we are done</font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font> pWAI <font color="#FF0000">!</font><font color="#FF0000">=</font> NULL <font color="#FF0000">)</font>
        pWAI<font color="#FF0000">-</font><font color="#FF0000">></font>Release<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>

    <font color="#006600">//We are done with COM</font>
    CoUninitialize<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
</pre>

<p>
Note, this object may be instanciated out of process or on a remote machine by using 
a Surrogate. The Surrogate apears in the process list as DLLHOST.EXE.
</p>            

<ul>
	<li>Open "OLE/COM Viewer".</li>
	<li>Expand the "All Objects" branch.</li>
	<li>Click on the object you want to Surrogate.</li>
	<li>Check the "Use Surrogate Process" on the Implementation Property page.</li>
	<li>Modify <code>CoCreateInstance(</code>, replace <code>CLSCTX_ALL</code> with <code>CLSCTX_LOCAL_SERVER</code>.</li>
	<li>This will result in a more robust object but will signifigantly reduce performance.</li>
	<li>If you want remote instanciate on another machine.</li>
	<ul>
		<li>Modify <code>CoCreateInstance(</code>, replace <code>CLSCTX_ALL</code> with <code>CLSCTX_REMOTE_SERVER</code>.</li>
		<li>In "OLE/COM Viewer" select the Activation Tab.</li>
		<li>Set the <u>R</u>emote Machine Name to the computer you want the object to be instanciated on.</li>
		<li>The object dll must be registered on that computer and set to run a a Surrogate.</li>
		<li>Under these conditions security is enabled by default so it may be nessary to se some permissions.</li>
	</ul>
</ul>

<!-- #include virtual="/Inc/NoteFooter.inc" --> 




