
<!-- #include virtual="/Inc/NoteHeader.inc" --> 


<h1>Description.</h1>
      
<p> There are two phases to the development of a database 
connection in Java. The first is to set up a DSN and the second is to write some 
code. I will cover both these steps. Note that this is written for the only operating system I know Microsoft Windows. If any one 
has another way to do this for better cross platform control,
I wold be happy to add your ideas to this article.</p>
   
<h1>Setup the DSN</h1>

<p>The following steps will setup a <em>System DSN</em> to a Microsoft Access database.</p>

<ol>
<li>Open the Windows Control Panel.
  <li>If running Windows 2000 open the <em>Administrative Tools</em> item.
  <li>Open the&nbsp;<em>Data Sources (ODBC)</em> item. 
  <li>Select the <em>System DSN</em> Tab.
  <li>Select <em>Add..</em>.
  <li>Double click on <em>Microsoft Access Driver (*.mdb)</em>.
  <li>Enter a name for the data source, <code>VisualCAccess</code>. 
  <li>Press <em>Select...</em>.
  <li>Browse for the database to open.
  <li>Press <em>OK</em>. That is it.</li>
</ol>


<h1>Write the code</h1>

<p>Each of the following examples shows how to append, read 
and modify the data. To use each of these steps it is necessary to include the followinf import.</p>

<pre>
<font color="#0088FF">import</font> java.sql.<font color="#FF0000">*</font><font color="#FF0000">;</font>
</pre>

<h2>Add a record</h2>

<p>Adding a record is done using the SQL <code>INSERT</code> command.</p>

<pre>
<font color="#0088FF">try</font> <font color="#FF0000">{</font>
    Class.forName<font color="#FF0000">(</font><font color="#FF00FF">"sun.jdbc.odbc.JdbcOdbcDriver"</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#006600">//OR Class.forName("com.ms.jdbc.odbc.JdbcOdbcDriver");</font>
            
    Connection conn <font color="#FF0000">=</font> DriverManager.getConnection<font color="#FF0000">(</font>
        <font color="#FF00FF">"JDBC:ODBC:Driver={Microsoft Access Driver};"</font><font color="#FF0000">+</font>
        <font color="#FF00FF">"dsn=VisualCAccess;UID=;PWD=;"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    
    Statement st <font color="#FF0000">=</font> conn.createStatement<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    java.util.Date dNow <font color="#FF0000">=</font> <font color="#0088FF">new</font> java.util.Date<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    java.text.DateFormat dFormat <font color="#FF0000">=</font> java.text.DateFormat.getDateTimeInstance<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    String sCommand <font color="#FF0000">=</font> <font color="#FF00FF">"INSERT INTO Hit "</font> <font color="#FF0000">+</font>
        <font color="#FF00FF">"(Path, IPAddress, BrowserString, HitTime) "</font> <font color="#FF0000">+</font>
        <font color="#FF00FF">"VALUES('AAA','10.1.1.1','Mosaic',"</font>
        <font color="#FF00FF">"'"</font> <font color="#FF0000">+</font> dFormat.format<font color="#FF0000">(</font> dNow <font color="#FF0000">)</font> <font color="#FF0000">+</font> <font color="#FF00FF">"' );"</font><font color="#FF0000">;</font>
    st.executeUpdate<font color="#FF0000">(</font> sCommand <font color="#FF0000">)</font><font color="#FF0000">;</font>
    st.close<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    conn.close<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
<font color="#FF0000">}</font> 
<font color="#0088FF">catch</font><font color="#FF0000">(</font> Exception ex <font color="#FF0000">)</font> <font color="#FF0000">{</font>
    String sErr <font color="#FF0000">=</font> ex.toString<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
<font color="#FF0000">}</font>
</pre>

<h2>Modifying a record</h2>

<p>Changing a record is very simular using the SQL <code>UPDATE</code> command.
Here record ID 11500 has its Wxyz set to 57, in the Hit table</p>

<pre>
<font color="#0088FF">try</font> <font color="#FF0000">{</font>
    Class.forName<font color="#FF0000">(</font><font color="#FF00FF">"sun.jdbc.odbc.JdbcOdbcDriver"</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#006600">//OR Class.forName("com.ms.jdbc.odbc.JdbcOdbcDriver");</font>
            
    Connection conn <font color="#FF0000">=</font> DriverManager.getConnection<font color="#FF0000">(</font>
        <font color="#FF00FF">"JDBC:ODBC:Driver={Microsoft Access Driver};"</font><font color="#FF0000">+</font>
        <font color="#FF00FF">"dsn=VisualCAccess;UID=;PWD=;"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    
    Statement st <font color="#FF0000">=</font> conn.createStatement<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    String sCommand <font color="#FF0000">=</font> <font color="#FF00FF">"UPDATE Hit SET Wxyz = 57 WHERE ID = 11500"</font><font color="#FF0000">;</font>
    st.executeUpdate<font color="#FF0000">(</font> sCommand <font color="#FF0000">)</font><font color="#FF0000">;</font>
    st.close<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    conn.close<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
<font color="#FF0000">}</font> 
<font color="#0088FF">catch</font><font color="#FF0000">(</font> Exception ex <font color="#FF0000">)</font> <font color="#FF0000">{</font>
    String sErr <font color="#FF0000">=</font> ex.toString<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
<font color="#FF0000">}</font>
</pre>


<h2>Reading a record</h2>

<p>Reading is a slightly different task. Here all records from the
Hit database will be read and displayed.</p>

<pre>
<font color="#0088FF">try</font> <font color="#FF0000">{</font>
    Class.forName<font color="#FF0000">(</font><font color="#FF00FF">"sun.jdbc.odbc.JdbcOdbcDriver"</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#006600">//OR Class.forName("com.ms.jdbc.odbc.JdbcOdbcDriver");</font>
            
    Connection conn <font color="#FF0000">=</font> DriverManager.getConnection<font color="#FF0000">(</font>
        <font color="#FF00FF">"JDBC:ODBC:Driver={Microsoft Access Driver};"</font><font color="#FF0000">+</font>
        <font color="#FF00FF">"dsn=VisualCAccess;UID=;PWD=;"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>

    Statement st <font color="#FF0000">=</font> conn.createStatement<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    ResultSet rs <font color="#FF0000">=</font> st.executeQuery<font color="#FF0000">(</font> <font color="#FF00FF">"SELECT * FROM Hit"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#0088FF">while</font><font color="#FF0000">(</font> rs.next<font color="#FF0000">(</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">{</font>
        String sPath <font color="#FF0000">=</font> rs.getString<font color="#FF0000">(</font> <font color="#FF00FF">"Path"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
        String sIP   <font color="#FF0000">=</font> rs.getString<font color="#FF0000">(</font> <font color="#FF00FF">"IPAddress"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
        String sBS   <font color="#FF0000">=</font> rs.getString<font color="#FF0000">(</font> <font color="#FF00FF">"BrowserString"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
        String sHT   <font color="#FF0000">=</font> rs.getString<font color="#FF0000">(</font> <font color="#FF00FF">"HitTime"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">long</font> nUserID <font color="#FF0000">=</font> rs.getLong  <font color="#FF0000">(</font> <font color="#FF00FF">"MemberID"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">long</font> nID     <font color="#FF0000">=</font> rs.getLong  <font color="#FF0000">(</font> <font color="#FF00FF">"ID"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>

   rs.close<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
   st.close<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
   conn.close<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
<font color="#FF0000">}</font> 
<font color="#0088FF">catch</font><font color="#FF0000">(</font> Exception ex <font color="#FF0000">)</font> <font color="#FF0000">{</font>
    String sErr <font color="#FF0000">=</font> ex.toString<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
<font color="#FF0000">}</font>
</pre>

<!-- #include virtual="/Inc/NoteFooter.inc" --> 
