


<!-- #include virtual="/Inc/NoteHeader.inc" --> 

<h4>Description.</h4>
      
<p>Polymorphism simply means creating an object with different forms. This
is useful when modeling real world items in software. For example a car
is a basic object but it can have many forms such as "Toyota", "Ford" and "Lada".
   
<h4>How to use it.</h4>

The following code demonstrates this by creating a base object, CAnimal as follows;

<pre>
<font color="#006600">//This is an abstract (base) class that all the amimals must support</font>
<font color="#0088FF">abstract</font> <font color="#0088FF">class</font> CAnimal
<font color="#FF0000">{</font>
    <font color="#0088FF">public</font> <font color="#0088FF">abstract</font> <font color="#0088FF">void</font> WhatAreYou<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#0088FF">public</font> <font color="#0088FF">void</font> Speak<font color="#FF0000">(</font><font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        System.out.print<font color="#FF0000">(</font> <font color="#FF00FF">"Quite"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
<font color="#FF0000">}</font>
</pre>

Note CAnimal uses the keyword <code>abstract</code> to indicating this is a base
class and cannot use to create an object directly. The <code>WhatAreYou();</code>
has not body and MUST be declared in all classes that are derived from it. The classes
below are derived from the CAnimal class.

<pre>
<font color="#006600">//This snake overides name and sound</font>
<font color="#0088FF">class</font> CSnake <font color="#0088FF">extends</font> CAnimal
<font color="#FF0000">{</font>
    <font color="#0088FF">public</font> <font color="#0088FF">void</font> WhatAreYou<font color="#FF0000">(</font><font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        System.out.print<font color="#FF0000">(</font> <font color="#FF00FF">"Snake"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
    <font color="#0088FF">public</font> <font color="#0088FF">void</font> Speak<font color="#FF0000">(</font><font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        System.out.print<font color="#FF0000">(</font> <font color="#FF00FF">"Sssssss"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
<font color="#FF0000">}</font>

<font color="#006600">//Monkey overides name but uses the default sound</font>
<font color="#0088FF">class</font> CMonkey <font color="#0088FF">extends</font> CAnimal
<font color="#FF0000">{</font>
    <font color="#0088FF">public</font> <font color="#0088FF">void</font> WhatAreYou<font color="#FF0000">(</font><font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        System.out.print<font color="#FF0000">(</font> <font color="#FF00FF">"Monkey"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
<font color="#FF0000">}</font>
</pre>

See how CSnake overides the <code>Speak()</code> method also. Now 
lets put it all together.

<pre>
<font color="#006600">///////////////////////////////////////////////////////////////////////////////</font>
<font color="#006600">// Program demonstrates a single class having diferent forms.</font>
<font color="#0088FF">public</font> <font color="#0088FF">class</font> Polymorph
<font color="#FF0000">{</font>
    <font color="#006600">// The main entry point for the application. </font>
    <font color="#006600">// @param args Array of parameters passed to the application</font>
    <font color="#0088FF">public</font> <font color="#0088FF">static</font> <font color="#0088FF">void</font> main <font color="#FF0000">(</font>String<font color="#FF0000">[</font><font color="#FF0000">]</font> args<font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        System.out.println<font color="#FF0000">(</font> <font color="#FF00FF">"Starting Polymorph application"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#006600">//Initialize array of zoo animals</font>
        CAnimal zoo<font color="#FF0000">[</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> <font color="#0088FF">new</font> CAnimal<font color="#FF0000">[</font><font color="#6600FF">2</font><font color="#FF0000">]</font><font color="#FF0000">;</font>
        zoo<font color="#FF0000">[</font><font color="#6600FF">0</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> <font color="#0088FF">new</font> CMonkey<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
        zoo<font color="#FF0000">[</font><font color="#6600FF">1</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> <font color="#0088FF">new</font> CSnake<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#006600">//Output results</font>
        <font color="#0088FF">for</font><font color="#FF0000">(</font> <font color="#0088FF">int</font> n <font color="#FF0000">=</font> <font color="#6600FF">0</font><font color="#FF0000">;</font> n <font color="#FF0000"><</font> zoo.length<font color="#FF0000">;</font> n <font color="#FF0000">+</font><font color="#FF0000">+</font> <font color="#FF0000">)</font>
        <font color="#FF0000">{</font>
            zoo<font color="#FF0000">[</font>n<font color="#FF0000">]</font>.WhatAreYou<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
            System.out.print<font color="#FF0000">(</font> <font color="#FF00FF">" says "</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
            zoo<font color="#FF0000">[</font>n<font color="#FF0000">]</font>.Speak<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
            System.out.println<font color="#FF0000">(</font> <font color="#FF00FF">"."</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#FF0000">}</font>
        System.out.println<font color="#FF0000">(</font> <font color="#FF00FF">"Terminating Polymorph application!"</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
<font color="#FF0000">}</font>
</pre>

The above code will give the following output.

<code>
Starting Polymorph application<br>
Monkey says Quite.<br>
Snake says Sssssss.<br>
Terminating Polymorph application!
</code>      
      

      
<!-- #include virtual="/Inc/NoteFooter.inc" --> 




