
<!-- #include virtual="/Inc/NoteHeader.inc" --> 


<h4>Description.</h4>
      
<p>A singleton is a object that is limited to creating only one
instance of itself. This can be useful when you want need to share
some data or a resource with other parts of your application.</p>
   
<h4>How to create a singleton.</h4>

<p>The following class implements a singleton with the following steps. First, make the 
constructor private. Second add a static instance of the class that is
initialised only once in the <code>getInstance()</code> where a reference
to the class is returned.</p>

<p>The variable <code>m_dCurrentRate</code> is only provided to 
show the functionality of the class.</p>

<pre>
<font color="#006600">/////////////////////////////////////////////////////////////////////////////</font>
<font color="#006600">//Class that only creates a single instance of itself</font>
<font color="#0088FF">class</font> TheSingleGuy
<font color="#FF0000">{</font>
    <font color="#006600">//Attributes</font>
    <font color="#0088FF">static</font> TheSingleGuy m_TheInstance<font color="#FF0000">;</font>
    <font color="#0088FF">private</font> <font color="#0088FF">double</font> m_dCurrentRate<font color="#FF0000">;</font>
    
    <font color="#006600">//Constructor is private so can only be made internally</font>
    <font color="#0088FF">private</font> TheSingleGuy<font color="#FF0000">(</font><font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        m_dCurrentRate <font color="#FF0000">=</font> <font color="#6600FF">100.0</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
    
    <font color="#006600">//Accessor and creator of the singleton</font>
    <font color="#0088FF">static</font> TheSingleGuy getInstance<font color="#FF0000">(</font><font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        <font color="#0088FF">if</font><font color="#FF0000">(</font> m_TheInstance <font color="#FF0000">=</font><font color="#FF0000">=</font> <font color="#0088FF">null</font> <font color="#FF0000">)</font>
            m_TheInstance <font color="#FF0000">=</font> <font color="#0088FF">new</font> TheSingleGuy<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">return</font> m_TheInstance<font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
    
    <font color="#006600">//Mutalators for the interest rate</font>
    <font color="#0088FF">void</font> setRate<font color="#FF0000">(</font> <font color="#0088FF">double</font> dRate <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        <font color="#0088FF">synchronized</font><font color="#FF0000">(</font> <font color="#0088FF">this</font> <font color="#FF0000">)</font>
        <font color="#FF0000">{</font>
            m_dCurrentRate <font color="#FF0000">=</font> dRate<font color="#FF0000">;</font>
        <font color="#FF0000">}</font>
    <font color="#FF0000">}</font>
    <font color="#0088FF">double</font> getRate<font color="#FF0000">(</font><font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        <font color="#0088FF">synchronized</font><font color="#FF0000">(</font> <font color="#0088FF">this</font> <font color="#FF0000">)</font>
        <font color="#FF0000">{</font>
            <font color="#0088FF">return</font> m_dCurrentRate<font color="#FF0000">;</font>
        <font color="#FF0000">}</font>
    <font color="#FF0000">}</font>
<font color="#FF0000">}</font>
</pre>

<p>To use this class do not call the constructor. Simple call
<code>getInstance()</code>  and use this value to access the class. Note 
in the example the rate is set for both instances to be the same.</p>

<p>Also note <code>synchronized( this )</code> is used. This is not necessary
in this example but should be considered if sharing a singleton 
across several threads.</p>

<pre>
    TheSingleGuy sg1 <font color="#FF0000">=</font> TheSingleGuy.getInstance<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    TheSingleGuy sg2 <font color="#FF0000">=</font> TheSingleGuy.getInstance<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    sg1.setRate<font color="#FF0000">(</font> <font color="#6600FF">200.0</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    sg2.setRate<font color="#FF0000">(</font> <font color="#6600FF">300.0</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    System.out.println<font color="#FF0000">(</font> <font color="#FF00FF">"Rate 1 = "</font> <font color="#FF0000">+</font> sg1.getRate<font color="#FF0000">(</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    System.out.println<font color="#FF0000">(</font> <font color="#FF00FF">"Rate 2 = "</font> <font color="#FF0000">+</font> sg2.getRate<font color="#FF0000">(</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
</pre>

<!-- #include virtual="/Inc/NoteFooter.inc" --> 




