
<!-- #include virtual="/Inc/NoteHeader.inc" -->


<h1>Introduction </h1>
<p>The document discusses the use of UNICODE to produce international (I18N) 
applications. This type of application is necessary when using 16 bit character 
sets such as those used in Chinese languages. </p>

<h1>Getting started</h1>
<p>To take advantage of the MFC and C run-time support for Unicode, you need to: 
</p>

<h2>Define _UNICODE</h2>
<p>Under C/C++ tab in Project settings, Category: General. Remove <code>_MBCS</code> and add 
<code>_UNICODE</code> to the list of Preprocessor Definitions. </p>

<h2>Specify entry point</h2>
<p>In the Output category of the Link tab in the Project Settings dialog box, 
set the Entry Point Symbol to <code>wWinMainCRTStartup</code>. </p>

<h2>Install correct librarys </h2>
<p>The default installtion of VC++ does not install the unicode librarys. To get 
these librarys to must run the Visual Studio setup and drill down into VC 
installed components to install the correct libraries.<br>
<UL type=circle>
	<LI>Microsoft Visual C++</LI>
	<LI>VC++ MFC and Template Libraries</LI>
	<LI>MS Foundation Class Libraries</LI>
	<LI>(** Get the lot **)</LI>
</UL>
</p>

<h2>Use "portable" run-time functions and types</h2>
<p>Use the proper C run-time functions for Unicode string handling. You 
can use the wcs family of functions, but you SHOULD use the fully "portable" 
(internationally enabled) <code>_TCHAR</code> macros. These macros are all prefixed with 
<code>_tcs</code>; they substitute, one for one, for the <code>str</code> family of functions. </p>

<h2>Handle literal strings properly</h2>
<p>The Visual C++ compiler interprets a literal string coded as <code>L"this is a 
literal string"</code> to mean a string of Unicode characters. You can use the same 
prefix for literal characters. Use the <code>_T</code> macro to code literal strings 
generically, so they compile as Unicode strings under Unicode or as ANSI strings 
(including MBCS) without Unicode. For example, instead of:<br> 
<code>&nbsp;&nbsp;&nbsp;&nbsp;pWnd-&gt;SetWindowText( "Hello" );</code> <br>
use:<br>
<code>&nbsp;&nbsp;&nbsp;&nbsp;pWnd-&gt;SetWindowText( _T("Hello") );</code><br> 
With <code>_UNICODE</code> defined, <code>_T</code> translates the literal string to the L-prefixed form; 
otherwise, <code>_T</code> translates the string without the <code>L</code> prefix.<br>
 Tip The <code>_T</code> macro is identical to the <code>_TEXT</code> macro. </p>

<h1>Displaying chinese charactors </h1>
<p>To display chinese charactors requires the correct font to be installed. Two 
methods are to be used depending on what operating system you are using. </p>

<h2>Running Chinese WinNT </h2>
<p>Under Chinese NT you simply create a UNICODE application 
is display a character such as;</p>
<pre>
WCHAR szOut<font color="#FF0000">[</font><font color="#6600FF">255</font><font color="#FF0000">]</font><font color="#FF0000">;</font>
szOut<font color="#FF0000">[</font><font color="#6600FF">0</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> <font color="#6600FF">0x58C6</font><font color="#FF0000">;</font>
szOut<font color="#FF0000">[</font><font color="#6600FF">1</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> <font color="#6600FF">0x6A4E</font><font color="#FF0000">;</font>
szOut<font color="#FF0000">[</font><font color="#6600FF">2</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> <font color="#6600FF">0x6FB8</font><font color="#FF0000">;</font>
szOut<font color="#FF0000">[</font><font color="#6600FF">3</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> <font color="#6600FF">0x5673</font><font color="#FF0000">;</font>
szOut<font color="#FF0000">[</font><font color="#6600FF">4</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> NULL<font color="#FF0000">;</font>
SetDlgItemText<font color="#FF0000">(</font> IDC_EDIT<font color="#FF0000">,</font> szOut <font color="#FF0000">)</font><font color="#FF0000">;</font> 
</pre>

<h2>Running non Chinese WinNT </h2>
<p>Under this operating system it is necessary to specifiy a 
font to display the chararctors as follows;
<pre><font color="#006600">//Get the default font height and Define new font</font>
CFont<font color="#FF0000">*</font> pfont <font color="#FF0000">=</font> GetDlgItem<font color="#FF0000">(</font> IDC_EDIT_IN <font color="#FF0000">)</font><font color="#FF0000">-</font><font color="#FF0000">></font>GetFont<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
LOGFONT LogFont<font color="#FF0000">;</font>pfont<font color="#FF0000">-</font><font color="#FF0000">></font>GetLogFont<font color="#FF0000">(</font> <font color="#FF0000">&</font>LogFont <font color="#FF0000">)</font><font color="#FF0000">;</font>
<font color="#0088FF">bool</font> b <font color="#FF0000">=</font> m_fontChinese.CreatePointFont<font color="#FF0000">(</font>
    LogFont.lfHeight<font color="#FF0000">*</font><font color="#FF0000">(</font><font color="#6600FF">10</font><font color="#FF0000">)</font><font color="#FF0000">,</font>  
    _T<font color="#FF0000">(</font><font color="#FF00FF">"MingLiU"</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
GetDlgItem<font color="#FF0000">(</font> IDC_EDIT <font color="#FF0000">)</font><font color="#FF0000">-</font><font color="#FF0000">></font>SetFont<font color="#FF0000">(</font> <font color="#FF0000">&</font>m_fontChinese <font color="#FF0000">)</font><font color="#FF0000">;</font>
<font color="#006600">//^^^^ Above code in InitDialog^^^^^</font>
    
<font color="#006600">//Some where in the application..</font>
WCHAR szOut<font color="#FF0000">[</font><font color="#6600FF">255</font><font color="#FF0000">]</font><font color="#FF0000">;</font>
szOut<font color="#FF0000">[</font><font color="#6600FF">0</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> <font color="#6600FF">0x58C6</font><font color="#FF0000">;</font>
szOut<font color="#FF0000">[</font><font color="#6600FF">1</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> <font color="#6600FF">0x6A4E</font><font color="#FF0000">;</font>
szOut<font color="#FF0000">[</font><font color="#6600FF">2</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> <font color="#6600FF">0x6FB8</font><font color="#FF0000">;</font>
szOut<font color="#FF0000">[</font><font color="#6600FF">3</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> <font color="#6600FF">0x5673</font><font color="#FF0000">;</font>
szOut<font color="#FF0000">[</font><font color="#6600FF">4</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> NULL<font color="#FF0000">;</font>
SetDlgItemText<font color="#FF0000">(</font> IDC_EDIT<font color="#FF0000">,</font> szOut <font color="#FF0000">)</font><font color="#FF0000">;</font>
</pre>

<h1>Using conversion Macros</h1>
<p>When using <code>WCHAR</code> or <code>TCHAR</code> and 
<code>char</code> strings together in the same application it is often necessary to 
convert between these types. This is often the case when data comes from
a socket connection or an ASCI file.</p>
<pre>
<font color="#0088FF">#include</font> <font color="#FF0000"><</font>atlbase.h<font color="#FF0000">></font>                 <font color="#006600">//This may not be necessary</font>
<font color="#0088FF">#include</font> <font color="#FF0000"><</font>atlconv.h<font color="#FF0000">></font>                 <font color="#006600">//..</font>
...
...
USES_CONVERSION<font color="#FF0000">;</font>
<font color="#0088FF">char</font> szSmall<font color="#FF0000">[</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> <font color="#FF00FF">"small string"</font><font color="#FF0000">;</font>
TCHAR<font color="#FF0000">*</font>  szBoth <font color="#FF0000">=</font> A2T<font color="#FF0000">(</font> szSmall <font color="#FF0000">)</font><font color="#FF0000">;</font>     <font color="#006600">//This is single byte for _MBCS</font>
LPCWSTR szWide <font color="#FF0000">=</font> A2CW<font color="#FF0000">(</font> szSmall <font color="#FF0000">)</font><font color="#FF0000">;</font>    <font color="#006600">//This is always double byte</font>
</pre>
<p>The above code takes an ANSI string and converts it into a char 
if compiled with <code>_MBCS</code> and into a wchar if compiled with
<code>_UNICODE</code>. The <code>USES_CONVERSIONS</code> macro ensures
the created string is cleaned up is necessary.</p>

<h1>Handy referances</h1>
<p>The following articles have some good information on this topic </p>
<p>
<table cellspacing=0 cellpadding=3 width="100%" border=1>
  <tr bgcolor="Navy">
    <td><strong><font color=white>Article ID</font></strong></td>
    <td><strong><font color=white>Title</font></strong> </td></tr>
  <tr>
    <td>Q138813</td>
    <td>How to Convert from ANSI to Unicode &amp; Unicode to ANSI for OLE 
  </td></tr>
  <tr>
    <td>TN059</td>
    <td>Using MFC MBCS/Unicode Conversion Macros</td></tr>
</table>



<!-- #include virtual="/Inc/NoteFooter.inc" -->
