


<!-- #include virtual="/Inc/NoteHeader.inc" --> 


<h4>Description.</h4>
      
<p>Threads are an important and tricky part of a multi tasking operating system. 
This article will show how to use threads and mutex but will not detail thread 
design issues such as Deadlocks and Spinlocks.</p>

<h4>How to start a thread.</h4>

<p>It is necessary to create a function to run when the thread starts. This
will continuously run until the function returns. The programmer should
either use a flag or counter to limit the number of times the process loops.</p>

<pre>
<font color="#006600">//Function to display the thread count 20 times and terminate</font>
UINT StartPlayingThread<font color="#FF0000">(</font>LPVOID param<font color="#FF0000">)</font>
<font color="#FF0000">{</font>
    AFX_MANAGE_STATE<font color="#FF0000">(</font> AfxGetAppModuleState<font color="#FF0000">(</font> <font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>    <font color="#006600">//Thread safe MFC access.</font>
    TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"StartPlayingThread(%p)\n"</font><font color="#FF0000">)</font><font color="#FF0000">,</font> param <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#0088FF">int</font> <font color="#FF0000">*</font>pnThreadNo <font color="#FF0000">=</font> <font color="#FF0000">(</font><font color="#0088FF">int</font><font color="#FF0000">*</font><font color="#FF0000">)</font>param<font color="#FF0000">;</font>
    ASSERT<font color="#FF0000">(</font> pnThreadNo <font color="#FF0000">)</font><font color="#FF0000">;</font>

    <font color="#006600">//Loop recording the count</font>
    <font color="#0088FF">for</font><font color="#FF0000">(</font> <font color="#0088FF">int</font> nCnt <font color="#FF0000">=</font> <font color="#6600FF">0</font><font color="#FF0000">;</font> nCnt <font color="#FF0000"><</font> <font color="#6600FF">20</font><font color="#FF0000">;</font> nCnt<font color="#FF0000">+</font><font color="#FF0000">+</font> <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"Count %d on thread %d\n"</font><font color="#FF0000">)</font><font color="#FF0000">,</font> nCnt<font color="#FF0000">,</font> <font color="#FF0000">*</font>pnThreadNo <font color="#FF0000">)</font><font color="#FF0000">;</font>
        Sleep<font color="#FF0000">(</font> <font color="#6600FF">1</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>

    <font color="#006600">//Clean up</font>
    <font color="#0088FF">delete</font> pnThreadNo<font color="#FF0000">;</font>
    <font color="#0088FF">return</font> <font color="#6600FF">0</font><font color="#FF0000">;</font>
<font color="#FF0000">}</font>
</pre>

<p>To launch the thread call <code>AfxBeginThread</code> with the starting function
name and one pointer. In this example the pointer is to an integer. This pointer
is a good way to share data with the thread. DO NOT pass
a pointer to a local variable as this may go out of scope and be deleted
before the thread that is using this memory terminates.</p>

<pre>
    <font color="#006600">//Run thread one</font>
    <font color="#0088FF">int</font> <font color="#FF0000">*</font>pNo <font color="#FF0000">=</font> <font color="#0088FF">new</font> <font color="#0088FF">int</font><font color="#FF0000">;</font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font> pNo <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        <font color="#FF0000">*</font>pNo <font color="#FF0000">=</font> <font color="#6600FF">1</font><font color="#FF0000">;</font>
        AfxBeginThread<font color="#FF0000">(</font> <font color="#FF0000">&</font>StartPlayingThread<font color="#FF0000">,</font> pNo <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>

    <font color="#006600">//Run thread two</font>
    <font color="#0088FF">int</font> <font color="#FF0000">*</font>pNo2 <font color="#FF0000">=</font> <font color="#0088FF">new</font> <font color="#0088FF">int</font><font color="#FF0000">;</font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font> pNo2 <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        <font color="#FF0000">*</font>pNo2 <font color="#FF0000">=</font> <font color="#6600FF">2</font><font color="#FF0000">;</font>
        AfxBeginThread<font color="#FF0000">(</font> <font color="#FF0000">&</font>StartPlayingThread<font color="#FF0000">,</font> pNo2 <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
</pre>

<p>In the above example the two threads will run in parallel. Run
in debug and view the trace log to see the output.</p>

<h4>Using Mutex.</h4>

<p>As not all objects are "thread safe" (able to be processed by two threads 
at the same time) it is often necessary to stop processing in a certain area 
until another thread is finished in that area. In the example that follows 
<code>g_nCounter</code> is incremented after an exact match test is 
performed. If both threads increment <code>g_nCounter</code> at the same time 
33 could be missed. <code>EnterCriticalSection</code> is used to hold other threads 
out of the testing/processing area until the thread is finished
in this area.</p>

<pre>
<font color="#006600">//Shared Global counter</font>
<font color="#0088FF">static</font> <font color="#0088FF">int</font> g_nCounter <font color="#FF0000">=</font> <font color="#6600FF">0</font><font color="#FF0000">;</font>

<font color="#006600">//Function to loop until global g_nCounter == 33</font>
UINT StartPlayingThread<font color="#FF0000">(</font>LPVOID param<font color="#FF0000">)</font>
<font color="#FF0000">{</font>
    AFX_MANAGE_STATE<font color="#FF0000">(</font> AfxGetAppModuleState<font color="#FF0000">(</font> <font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>    <font color="#006600">//Thread safe MFC access.</font>
    TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"StartPlayingThread(%p)\n"</font><font color="#FF0000">)</font><font color="#FF0000">,</font> param <font color="#FF0000">)</font><font color="#FF0000">;</font>
    LPCRITICAL_SECTION lpCritSection <font color="#FF0000">=</font> <font color="#FF0000">(</font>LPCRITICAL_SECTION<font color="#FF0000">)</font>param<font color="#FF0000">;</font>
    ASSERT<font color="#FF0000">(</font> lpCritSection <font color="#FF0000">)</font><font color="#FF0000">;</font>

    <font color="#006600">//Loop recording the count</font>
    <font color="#0088FF">bool</font> bRun <font color="#FF0000">=</font> <font color="#0088FF">true</font><font color="#FF0000">;</font>
    <font color="#0088FF">while</font><font color="#FF0000">(</font> bRun <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        EnterCriticalSection<font color="#FF0000">(</font> lpCritSection <font color="#FF0000">)</font><font color="#FF0000">;</font>      <font color="#006600">//Only one thread in here</font>
        <font color="#0088FF">if</font><font color="#FF0000">(</font> g_nCounter <font color="#FF0000">=</font><font color="#FF0000">=</font> <font color="#6600FF">33</font> <font color="#FF0000">)</font>
            bRun <font color="#FF0000">=</font> <font color="#0088FF">false</font><font color="#FF0000">;</font>
        <font color="#0088FF">else</font>
            g_nCounter<font color="#FF0000">+</font><font color="#FF0000">+</font><font color="#FF0000">;</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"Thread ID=%ld, Count=%d\n"</font><font color="#FF0000">)</font><font color="#FF0000">,</font> AfxGetThread<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">-</font><font color="#FF0000">></font>m_nThreadID<font color="#FF0000">,</font> g_nCounter <font color="#FF0000">)</font><font color="#FF0000">;</font>
        LeaveCriticalSection<font color="#FF0000">(</font> lpCritSection <font color="#FF0000">)</font><font color="#FF0000">;</font>
        Sleep<font color="#FF0000">(</font> <font color="#6600FF">1</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
   <font color="#0088FF">return</font> <font color="#6600FF">0</font><font color="#FF0000">;</font>
<font color="#FF0000">}</font>
</pre>

<p>The following code creates a Critical Section handler to 
be shared by two threads that increment a shared integer resource.
Once the threads are started they are monitored until they
exit or a 5 second timer elapses.</p>

<pre>
   <font color="#006600">//Setup the mutex handler</font>
    CRITICAL_SECTION CriticalSection<font color="#FF0000">;</font>
    InitializeCriticalSection<font color="#FF0000">(</font> <font color="#FF0000">&</font>CriticalSection <font color="#FF0000">)</font><font color="#FF0000">;</font>

    <font color="#006600">//Run thread one</font>
    CWinThread<font color="#FF0000">*</font> wt1 <font color="#FF0000">=</font> AfxBeginThread<font color="#FF0000">(</font> <font color="#FF0000">&</font>StartPlayingThread<font color="#FF0000">,</font> <font color="#FF0000">&</font>CriticalSection <font color="#FF0000">)</font><font color="#FF0000">;</font>
    CWinThread<font color="#FF0000">*</font> wt2 <font color="#FF0000">=</font> AfxBeginThread<font color="#FF0000">(</font> <font color="#FF0000">&</font>StartPlayingThread<font color="#FF0000">,</font> <font color="#FF0000">&</font>CriticalSection <font color="#FF0000">)</font><font color="#FF0000">;</font>

    <font color="#006600">//Wait for the threads to terminate in 5-10 seconds</font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font> WaitForSingleObject<font color="#FF0000">(</font> wt1<font color="#FF0000">-</font><font color="#FF0000">></font>m_hThread<font color="#FF0000">,</font> <font color="#6600FF">5000</font> <font color="#FF0000">)</font> <font color="#FF0000">=</font><font color="#FF0000">=</font> WAIT_TIMEOUT <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"ERROR : Thread wt1. Shut Down Failed!!\n"</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
        exit<font color="#FF0000">(</font> <font color="#6600FF">0</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font> WaitForSingleObject<font color="#FF0000">(</font> wt2<font color="#FF0000">-</font><font color="#FF0000">></font>m_hThread<font color="#FF0000">,</font> <font color="#6600FF">5000</font> <font color="#FF0000">)</font> <font color="#FF0000">=</font><font color="#FF0000">=</font> WAIT_TIMEOUT <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"ERROR : Thread wt2. Shut Down Failed!!\n"</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
        exit<font color="#FF0000">(</font> <font color="#6600FF">0</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>

    <font color="#006600">//Clean up</font>
    TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"Threads exited OK!\n"</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    DeleteCriticalSection<font color="#FF0000">(</font> <font color="#FF0000">&</font>CriticalSection <font color="#FF0000">)</font><font color="#FF0000">;</font>
</pre>

<p>NOTE: It is important to note that it is not possible (to my knowledge)
to terminate a thread other than gracefully exiting thread function using software methods
or terminating the application. In the previous example if 5 seconds has passed
and the thread has not terminated it will NOT be forced to shut down.
For this reason it is important to plan how a the thread is going
to communicate and be terminated when designing the application. It is also
necessary to test the program on multi-processor systems which may process
threads differently that single processor machines.</p>

<!-- #include virtual="/Inc/NoteFooter.inc" --> 




