namespace Net 
{
    using System;
    using System.Collections;
    using System.ComponentModel;
    using System.Web;
    using System.Web.SessionState;
	using System.Data.ADO;						//Database
	using System.Globalization;					//Date
	using System.Diagnostics;					//Event log

    /// <summary>
    ///    This is the Global.asax implementation. It performs the following tasks.
    ///    * Contains some global definitions and handy gllobal functions.
    ///    * Record totals of ASPX pages visited.
    ///    * Record member details.
    /// </summary>
    public class Global : System.Web.HttpApplication
    {

		/// <value>DB_CONN_STRING Database connection string</value>
		public const string DB_CONN_STRING = 
			"Driver={Microsoft Access Driver (*.mdb)}; DBQ=C:\\Inetpub\\Private\\WebData.mdb"; 
		/// <value>BUILD_NO Build number for this library</value>
		public const short  BUILD_NO = 1004;


		private void InitializeComponent ()
		{
		}

        protected void Application_Start(Object sender, EventArgs e)
        {
			LogInformation( "Application START");
        }
		protected void Application_End(Object sender, EventArgs e)
        {
			LogInformation( "Application END.");
        }

        protected void Session_Start(Object sender, EventArgs e)
        {
			MemberDetails md = new MemberDetails( Session );
			md.Read( Request, Response );
			RecordVisitor();
        }
		protected void Session_End(Object sender, EventArgs e)
        {
        }

        protected void Application_BeginRequest(Object sender, EventArgs e)
        {
			RecordASPX_Hit();
        }

        protected void Application_EndRequest(Object sender, EventArgs e)
        {
        }

   
		/// <summary>
		///		Record information about the visitor. Information about callers
		///		IP address, Inital path, intended Host, Browser information.
		/// </summary>
		private void RecordVisitor()
		{  
			//Get and formath the current time to the local system
			DateTime dtNow = DateTime.Now;
			string sNow = dtNow.Format( "G", DateTimeFormatInfo.CurrentInfo );

			//Build the insert string
			String sCommand = "INSERT INTO HitSite (IPAddress, ArrivalPath, Host, BrowserString, HitTime, MemberEmail)  " +
   				"VALUES(" +
				"'" + Request.UserHostAddress					+ "', " +
				"'" + Request.FilePath							+ "', " +
				"'" + Request.Headers.Get( "Host" )				+ "', " +
				"'" + Request.UserAgent							+ "', " +
 				"'" + sNow										+ "', " +
 				"'" + ((MemberDetails)Session["Member"]).EMail	+ "' );";

			try
			{
				ADOCommand cmdAdder = new ADOCommand(
						sCommand,
						DB_CONN_STRING);
				cmdAdder.ActiveConnection.Open();
				cmdAdder.ExecuteNonQuery();
			}
			catch (Exception ex)
			{
				LogError("Record Visitor\n" + 
					"  ERROR:" + ex.Message + "\n" +
					"  Cmd. :" + sCommand + "\n" + 
					"  Now  :" + sNow + "\n" + 
					"  Conn.:" + DB_CONN_STRING );
			}
		}


		/// <summary>
		///		Record the page hit in the HitPage table for this month.
		///		* Open the Hit page Table.
		///		* Select the column for this month.
		///		* Increment the total or create the row.
		/// </summary>
        private void RecordASPX_Hit()
        {
			string sSqlCmd = "";
			string sPath = Request.RawUrl;
			DateTime dtNow = DateTime.Now;
			string sYearMonth = "Y" + 
					dtNow.Year + "_" + 
					dtNow.Month.Format( "d2", null );
			try
			{
				//Get the hit count if it exists
				ADODataReader dr;
				ADOConnection conn = new ADOConnection(DB_CONN_STRING);
				conn.Open();
				sSqlCmd = "SELECT " + sYearMonth + " FROM HitPage WHERE Path = " +
					"'" + sPath + "'";
				ADOCommand cmd = new ADOCommand( sSqlCmd, conn );
				cmd.Execute( out dr);
				// Always call Read before accessing data.
				if( dr.Read() )
				{
					int nHits = 0;
					if( !dr.IsNull( 0 ) )
					{
						nHits = dr.GetInt16( 0 ) + 1;
					}
					sSqlCmd = "UPDATE HitPage SET " + sYearMonth + 
							" = " + nHits +
							" WHERE Path = " +
							"'" + sPath + "'";			
				}
				else
				{
					sSqlCmd = "INSERT INTO HitPage (Path," + sYearMonth + 
						")  VALUES(" +
						"'" + sPath + "'," +
						"'" + 1 + "' );";				
				}
				dr.Close();
				cmd = new ADOCommand( sSqlCmd, conn );
				cmd.ExecuteNonQuery();
			}
			catch( Exception ex )
			{
				LogError("Application_EndRequest\n" + 
					"  ERROR:" + ex.Message + "\n" +
					"  Path :" + sPath + "\n" + 
					"  SQL  :" + sSqlCmd + "\n" + 
					"  Conn.:" + DB_CONN_STRING );
			}
        }


		///	<summary>
		///		Log information and events to the NT Application EventLog.
		///		<param name='sInfo'>Information for the log</param>
		///	</summary>
		public static void LogInformation( string sInfo ) 
		{
			LogEvent( sInfo, EventLogEntryType.Information );
		}

		/// <summary>
		///		Record an ERROR in the NT event log.
		/// </summary>
		/// <param name='sError'>Error information for the log.</param>
		public static void LogError( string sError )
		{
			LogEvent( sError, EventLogEntryType.Error );
		}
		private static short s_nEventCount = 0;
		private static EventLog s_eLog = null;

		/// <summary>
		///		This is the actual worker that records information about the event.
		/// </summary>
		/// <param name='sError'>Message body for the log.</param>
		/// <param name='elType'>Type of log message to record</param>
		private static void LogEvent( string sError, EventLogEntryType elType )
		{		
			//Create the log connection if necessary
			if( s_eLog == null )
			{
				const string LOG_SOURCE_NAME = "JrmWebServer01";
				if( !EventLog.SourceExists( LOG_SOURCE_NAME ) )
				{
					EventLog.CreateEventSource( LOG_SOURCE_NAME, "" );
				}
				s_eLog = new EventLog();
				s_eLog.Source = LOG_SOURCE_NAME;
			}
			s_eLog.WriteEntry( 
				sError, 
				elType,
				BUILD_NO,
                s_nEventCount++ );
		}
	}
}

