namespace Net
{
    using System;
	using System.Web;
	using System.Text;
	using System.Data.ADO;						//Database
	using System.Globalization;					//Date

    /// <summary>
    /// This class handles most of the operations associated with pages
    /// that display coding ideas and turorials. It does not include the
    /// section page, search or member page code.
    /// </summary>
    public class PageData
    {
		//Attributes
		private string m_sDbPath;				//Path in terms of database offset.
		private int    m_nID;
		private string m_sTitle;
		private string m_sSummary;
		private string m_sContrubutor;
		private string m_sEmail;
		private string m_sPublished;
		private string m_sUpdated;
		private int m_nVoteNo;
		private int m_nVoteTotal;

		//Property readers ( makes property read only )
		public int    ID			{ get { return m_nID;  }  }			//Page ID
		public string Title			{ get { return m_sTitle;  }  }		//Title of the page
		public string Summary		{ get { return m_sSummary;  }  }	//Page summary
		public string Contrubutor	{ get { return m_sContrubutor;  }  }
		public string Email			{ get { return m_sEmail;  }  }
		public string Published		{ get { return m_sPublished;  }  }
		public string Updated		{ get { return m_sUpdated;  }  }
	
		/// <summary>
		/// Constructor
		/// </summary>
        public PageData( string sPathInfo )
        {
  			m_sDbPath = sPathInfo;
//			int nPrefixLen = Section.SW_ROOT.Length;
//			if( nPrefixLen < sPathInfo.Length )
//			{
//				m_sDbPath = sPathInfo.Remove( 0, nPrefixLen );
//			}
      }

		/// <summary>
		/// Read the page setting information
		/// </summary>
		/// <param name="sPathInfo">Path of file being read</param>
		public void ReadFor()
		{
			string sSqlCmd = "SELECT * FROM Page WHERE Path = '" + m_sDbPath + "'";
			try
			{
				//Open the base on that line
				ADODataReader dr;
				ADOConnection conn = new ADOConnection(Global.DB_CONN_STRING);
				conn.Open();
				ADOCommand cmd = new ADOCommand( sSqlCmd, conn );
				cmd.Execute( out dr);
				// Always call Read before accessing data.
				if( dr.Read() )
				{
					DateTime datePublished = (DateTime)dr["Published"];
					DateTime dateUpdated   = (DateTime)dr["Updated"];

					m_nID			= (int)dr["ID"];
					m_sTitle		= dr["Title"].ToString();
					m_sSummary		= dr["Summary"].ToString();
					m_sContrubutor	= dr["Contrubutor"].ToString();
					m_sEmail		= dr["Email"].ToString();
					m_sPublished	= datePublished.Format( "d-MMM-yyyy", DateTimeFormatInfo.InvariantInfo );
					m_sUpdated		= dateUpdated.Format( "d-MMM-yyyy", DateTimeFormatInfo.InvariantInfo );
					m_nVoteNo		= (int)dr["VoteNo"];
					m_nVoteTotal	= (int)dr["VoteTotal"];
				}

				dr.Close();
			}
			catch( Exception ex )
			{
				Global.LogError("PageDate.ReadFor\n" + 
					"  ERROR:" + ex.Message + "\n" +
					"  Path :" + m_sDbPath + "\n" + 
					"  SQL  :" + sSqlCmd + "\n" + 
					"  Conn.:" + Global.DB_CONN_STRING );
			}
		}
	
		/// <summary>
		/// Update the hit counters
		/// NOTE: Read For MUST becalled befor this
		/// </summary>
		/// <param name="sRattedAs">integer as string indiicating amount to add to total. </param>
		public void RatePage( string sRattedAs ) 
		{
			m_nVoteNo    += 1;
			m_nVoteTotal += Int32.Parse( sRattedAs );

			//Update the Member database info
			string sSqlCmd = "UPDATE Page" +
						" SET   Page.VoteNo    = " + m_nVoteNo + ", " +
						"       Page.VoteTotal = " + m_nVoteTotal + 
						" WHERE Page.Path      = '"+ m_sDbPath +"';";			
			try
			{
				ADOConnection conn = new ADOConnection(Global.DB_CONN_STRING);
				conn.Open();
				ADOCommand cmd = new ADOCommand( sSqlCmd, conn );
				cmd.ExecuteNonQuery();
			}
			catch( Exception ex )
			{
				Global.LogError("PageData.RatePage\n" + 
					"  ERROR:" + ex.Message + "\n" +
					"  Rate :" + sRattedAs + "\n" + 
					"  SQL  :" + sSqlCmd + "\n" + 
					"  Conn.:" + Global.DB_CONN_STRING );
			}
		}

		/// <summary>
		/// Add an a comment to thhe comments database linked to the page
		/// page info database.
		/// </summary>
		/// <param name="Request">Request information</param>
		public void AddComment(  HttpRequest Request ) 
		{

			int nPageID		= Int32.Parse( Request.Form["PageID"] );
			string sName	= Request.Form["Name"];
			string sEmail	= Request.Form["Email"];
			string sTitle	= MakeSafeString( Request.Form["Title"] );
			string sNote	= MakeSafeString( Request.Form["Memo"] );
			DateTime dtNow = DateTime.Now;
			string sNow = dtNow.Format( "G", DateTimeFormatInfo.CurrentInfo );

			//Update the Member database info
			string sSqlCmd = 
				"INSERT INTO Comment "+
				"(PageID,Author,Email,Subject,Comment,TimeOfComment)"+
				" VALUES ("+nPageID+",'"+sName+"','"+sEmail+"','"+
				sTitle+"','"+sNote+"','"+sNow+"')"; 
			try
			{
				ADOConnection conn = new ADOConnection(Global.DB_CONN_STRING);
				conn.Open();
				ADOCommand cmd = new ADOCommand( sSqlCmd, conn );
				cmd.ExecuteNonQuery();
			}
			catch( Exception ex )
			{
				Global.LogError("PageData.AddComment\n" + 
					"  ERROR:" + ex.Message + "\n" +
					"  SQL  :" + sSqlCmd + "\n" + 
					"  Conn.:" + Global.DB_CONN_STRING );
			}
		}


		/// <summary>
		/// Remove any funny codes and convert LF into HTML. This is necessary when
		/// the user enters data that will be rendered to the display.
		/// < and > are removed and LF ar converted to <br>.
		/// </summary>
		/// <param name="sIn">String to convert</param>
		private string MakeSafeString( string sIn )
		{
			StringBuilder sb = new StringBuilder( sIn + " " );
			sb = sb.Replace( ">", "&gt;" );
			sb = sb.Replace( "<", "&lt;" );
			sb = sb.Replace( "\n", "<br>" );
			return sb.ToString();
		}


		/// <summary>
		/// Build an HTML string depending on the page rating
		/// </summary>
		public string GetRatingHtml()		
		{ 
			if( m_nVoteNo <= 0 )
			{
				return "Be the first to rate this page!";
			}
			int nPercent = m_nVoteTotal*20/m_nVoteNo;
			string s;
			s = "<table border=1 cellspacing=0 cellpadding=0 width=110 height=5 bgcolor=white bordercolor=black frame=border>";  
			s += "<tr height=8>";

			int nSize = Math.Min( 20, ( nPercent ) );
			s += "	<td width=20><img src='/images/Red.bmp' width="+nSize+" height=8></td>";

			if( nPercent < 20 )
			{
				s += "	<td width=20></td>";
			}
			else
			{
				nSize = Math.Min( 20, ( nPercent - 20 ) );
				s += "	<td width=20><img src='/images/Red.bmp' width="+nSize+" height=8></td>";
			}
			if( nPercent < 40 )
			{
				s += "	<td width=20></td>";
			}
			else
			{
				nSize = Math.Min( 20, ( nPercent - 40 ) );
				s += "	<td width=20><img src='/images/Red.bmp' width="+nSize+" height=8></td>";
			}
			if( nPercent < 60 )
			{
				s += "	<td width=20></td>";
			}
			else
			{
				nSize = Math.Min( 20, ( nPercent - 60 ) );
				s += "	<td width=20><img src='/images/Red.bmp' width="+nSize+" height=8></td>";
			}

			if( nPercent < 80 )
			{
				s += "	<td width=20></td>";
			}
			else
			{
				nSize = Math.Min( 20, ( nPercent - 80 ) );
				s += "	<td width=20><img src='/images/Red.bmp' width="+nSize+" height=8></td>";
			}
			s += "</tr>";
			s += "</table>";
			s += "<td nowrap bgcolor='#3399ff'>"+nPercent+"% for "+m_nVoteNo+" votes</td>";
			return s;
		}


		/// <summary>
		/// Take the comments from the given page and build them into a table for
		/// display at the bottom of the page. 
		/// MUST call ReadFor() before this.
		/// </summary>
		public string CommentsToHtml()
		{	
			string sOut = "";
			string sSqlCmd = "SELECT * FROM Comment WHERE Comment.PageID = " + m_nID + ";";
			try
			{
				//Open the base on that line
				ADODataReader dr;
				ADOConnection conn = new ADOConnection(Global.DB_CONN_STRING);
				conn.Open();
				ADOCommand cmd = new ADOCommand( sSqlCmd, conn );
				cmd.Execute( out dr);
				// Read in each comment
				while( dr.Read() )
				{
					string sAuthor		= dr["Author"].ToString();
					string sEmail		= dr["Email"].ToString();
					string sSubject		= dr["Subject"].ToString();
					string sComment		= dr["Comment"].ToString();
					DateTime dateAdded	= (DateTime)dr[ "TimeOfComment" ];
					string sDateAdded	= dateAdded.Format( "d-MMM-yyyy", DateTimeFormatInfo.InvariantInfo );

					//Output in table format
					sOut += "<tr>";
					sOut += "	<td bgcolor=#0066FF>" + sSubject + "</td>";
					sOut += "	<td bgcolor=#0066FF>"+sDateAdded+"</td>";
					sOut += "	<td bgcolor=#0066FF>";
					sOut += "		&nbsp;<A HREF='mailto:"+sEmail+"'>"+sAuthor+"</A></td>";
					sOut += "</tr>";
					sOut += "<tr>";
					sOut += "	<td colspan=3>"+sComment+"</td>";
					sOut += "</tr>";
					sOut += "<tr></tr>";
				}

				dr.Close();
			}
			catch( Exception ex )
			{
				Global.LogError("PageDate.WriteComments\n" + 
					"  ERROR:" + ex.Message + "\n" +
					"  Path :" + m_sDbPath + "\n" + 
					"  SQL  :" + sSqlCmd + "\n" + 
					"  Conn.:" + Global.DB_CONN_STRING );
			}
			return sOut;
		}
	}
}
