


<!-- #include virtual="/Inc/NoteHeader.inc" --> 

      <h4>Description.</h4>
      
      <p>The basic idea is to create an event that is attached to a registry
      key. When the key changes the event is triggered. It is then a simple task
      to poll the event to see if it has signalled and act upon this value.
      
      <h4>How to do it.</h4>
      
      Open the registry key and attach an event to it.

<pre>
    <font color="#006600">// Open a key.</font>
    TCHAR szSubKey<font color="#FF0000">[</font><font color="#FF0000">]</font> <font color="#FF0000">=</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"Software\\Microsoft\\Clock"</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    HKEY   hKey<font color="#FF0000">;</font>          <font color="#006600">// Key to the registery item to watch</font>
    LONG   lErrorCode<font color="#FF0000">;</font>    <font color="#006600">// Error code catcher</font>
    lErrorCode <font color="#FF0000">=</font> RegOpenKeyEx<font color="#FF0000">(</font>HKEY_CURRENT_USER<font color="#FF0000">,</font> 
                              szSubKey<font color="#FF0000">,</font> <font color="#6600FF">0</font><font color="#FF0000">,</font> KEY_NOTIFY<font color="#FF0000">,</font> <font color="#FF0000">&</font>hKey<font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font> lErrorCode <font color="#FF0000">!</font><font color="#FF0000">=</font> ERROR_SUCCESS <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"Error in RegOpenKeyEx.\n"</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">return</font> <font color="#0088FF">false</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
    
    <font color="#006600">// Create an event.</font>
    HANDLE hEvent<font color="#FF0000">;</font>        <font color="#006600">// Event output for change</font>
    hEvent <font color="#FF0000">=</font> CreateEvent<font color="#FF0000">(</font>NULL<font color="#FF0000">,</font> TRUE<font color="#FF0000">,</font> FALSE<font color="#FF0000">,</font> NULL<font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font> hEvent <font color="#FF0000">=</font><font color="#FF0000">=</font> NULL <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"Error in CreateEvent.\n"</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">return</font> <font color="#0088FF">false</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
    

    <font color="#006600">// Watch the registry key for a change of value.</font>
    DWORD  dwFilter <font color="#FF0000">=</font> REG_NOTIFY_CHANGE_NAME <font color="#FF0000">|</font>
                      REG_NOTIFY_CHANGE_ATTRIBUTES <font color="#FF0000">|</font>
                      REG_NOTIFY_CHANGE_LAST_SET <font color="#FF0000">|</font>
                      REG_NOTIFY_CHANGE_SECURITY<font color="#FF0000">;</font> 

    lErrorCode <font color="#FF0000">=</font> RegNotifyChangeKeyValue<font color="#FF0000">(</font>
        hKey<font color="#FF0000">,</font>            <font color="#006600">// handle to key to watch</font>
        TRUE<font color="#FF0000">,</font>            <font color="#006600">// subkey notification option</font>
        dwFilter<font color="#FF0000">,</font>        <font color="#006600">// changes to be reported</font>
        hEvent<font color="#FF0000">,</font>          <font color="#006600">// handle to event to be signaled</font>
        TRUE<font color="#FF0000">)</font><font color="#FF0000">;</font>           <font color="#006600">// asynchronous reporting option</font>

    <font color="#0088FF">if</font><font color="#FF0000">(</font> lErrorCode <font color="#FF0000">!</font><font color="#FF0000">=</font> ERROR_SUCCESS <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"Error in RegNotifyChangeKeyValue.\n"</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
        <font color="#0088FF">return</font> <font color="#0088FF">false</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
</pre>
      
     Where you want to check it the registry item has changed use.
          
     
<pre>
    <font color="#006600">// Check if the event has occured.</font>
    <font color="#0088FF">switch</font><font color="#FF0000">(</font> WaitForSingleObject<font color="#FF0000">(</font> hEvent<font color="#FF0000">,</font> <font color="#6600FF">0</font> <font color="#FF0000">)</font> <font color="#FF0000">)</font>
    <font color="#FF0000">{</font>
    <font color="#006600">// Reg item has changed</font>
    <font color="#0088FF">case</font> WAIT_OBJECT_0<font color="#FF0000">:</font>
        <font color="#0088FF">break</font><font color="#FF0000">;</font>
    <font color="#006600">// Reg item has NOT changed</font>
    <font color="#0088FF">case</font> WAIT_TIMEOUT<font color="#FF0000">:</font>
        <font color="#0088FF">break</font><font color="#FF0000">;</font>
    <font color="#006600">// This an error condition that should not happen</font>
    <font color="#006600">// if it does we best reopen the event and key.</font>
    <font color="#0088FF">default</font><font color="#FF0000">:</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"Error in WaitForSingleObject.\n"</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
</pre>     

		Before you go home clean up the mess.

<pre>
    <font color="#006600">// Close the key.</font>
    lErrorCode <font color="#FF0000">=</font> RegCloseKey<font color="#FF0000">(</font>hKey<font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font> lErrorCode <font color="#FF0000">!</font><font color="#FF0000">=</font> ERROR_SUCCESS <font color="#FF0000">)</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"Error in RegCloseKey.\n"</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    
    <font color="#006600">// Close the handle.</font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font> <font color="#FF0000">!</font>CloseHandle<font color="#FF0000">(</font> hEvent <font color="#FF0000">)</font> <font color="#FF0000">)</font>
        TRACE<font color="#FF0000">(</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"Error in CloseHandle.\n"</font><font color="#FF0000">)</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
</pre>

<!-- #include virtual="/Inc/NoteFooter.inc" --> 




