


<!-- #include virtual="/Inc/NoteHeader.inc" --> 

	<h4>Description.</h4>
      
		      
		<p>Sometimes more than one instance of an application can cause problems
		with resources such as multi media players and socket servers. To overcome
		this it is first necessary to detect if the application is already running
		and secondly activate the application to allowing the user to do what they
		initially wanted.</p>      
      
      
    <h4>Detecting the other installation.</h4>
      
		To detect if the other application is running we simply create a unique
		event and detect if it has already been created indicating the application
		is already running.
		<p>
		Where you want to detect if the other application is running, use the
		following code. I like to do this in <code>CMyApp::InitInstance()</code>.
		<p>
<pre>
<font color="#006600">//Check if another application is already running and note start</font>
LPCTSTR szEventName <font color="#FF0000">=</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"PtkBneReadyRefEvent"</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
<font color="#0088FF">if</font><font color="#FF0000">(</font> OpenEvent<font color="#FF0000">(</font> EVENT_ALL_ACCESS<font color="#FF0000">,</font> <font color="#0088FF">true</font><font color="#FF0000">,</font> szEventName <font color="#FF0000">)</font> <font color="#FF0000">)</font>
<font color="#FF0000">{</font>
    <font color="#0088FF">return</font> <font color="#0088FF">false</font><font color="#FF0000">;</font>
<font color="#FF0000">}</font>
CreateEvent<font color="#FF0000">(</font> NULL<font color="#FF0000">,</font> <font color="#0088FF">false</font><font color="#FF0000">,</font> <font color="#0088FF">false</font><font color="#FF0000">,</font> szEventName <font color="#FF0000">)</font><font color="#FF0000">;</font>
</pre>    
     
		<br>
	
		If you want to be picky Bounds Checker will say the event was not freed,
		to prevent this you may save the handle and destroy it in the destructor
		with <code>CloseHandle</code>.
	
	<h4>Detecting and Restoring the other installation.</h4>Preventing the
     
		application starting is only half the problem, if you just do
		this the user may get frustrated when the application does not appear.
		To overcome this force it to appear by posting a message to it.
		<br>
		<br>
		Create a member variable <code>UINT m_OtherAppMsgId</code> to hold a global window 
		message. Set it to NULL in the constructor and initialise it as
		below in <code>CMyApp::InitInstance()</code>. This will create the message and 
		post it of to the already running instance if it exists.
		
<pre>
<font color="#006600">//Check if another application is already running and note start</font>
LPCTSTR szEventName <font color="#FF0000">=</font> _T<font color="#FF0000">(</font><font color="#FF00FF">"PtkBneReadyRefEvent"</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
m_OtherAppMsgId <font color="#FF0000">=</font> <font color="#FF0000">:</font><font color="#FF0000">:</font>RegisterWindowMessage<font color="#FF0000">(</font><font color="#FF00FF">"SingleInstanceOfReadyRefApp"</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
<font color="#0088FF">if</font><font color="#FF0000">(</font> OpenEvent<font color="#FF0000">(</font> EVENT_ALL_ACCESS<font color="#FF0000">,</font> <font color="#0088FF">true</font><font color="#FF0000">,</font> szEventName <font color="#FF0000">)</font> <font color="#FF0000">)</font>
<font color="#FF0000">{</font>
    <font color="#006600">//Another application exists so get the other to activate.</font>
    <font color="#FF0000">:</font><font color="#FF0000">:</font>PostMessage<font color="#FF0000">(</font> HWND_BROADCAST<font color="#FF0000">,</font> m_OtherAppMsgId<font color="#FF0000">,</font> <font color="#6600FF">0</font><font color="#FF0000">,</font> <font color="#6600FF">0L</font> <font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#0088FF">return</font> <font color="#0088FF">false</font><font color="#FF0000">;</font>
<font color="#FF0000">}</font>
CreateEvent<font color="#FF0000">(</font> NULL<font color="#FF0000">,</font> <font color="#0088FF">false</font><font color="#FF0000">,</font> <font color="#0088FF">false</font><font color="#FF0000">,</font> szEventName <font color="#FF0000">)</font><font color="#FF0000">;</font>
</pre>

		Add the <code>PreTranslateMessage</code> message handller as follows.
	
<pre>
 BOOL CMyApp<font color="#FF0000">:</font><font color="#FF0000">:</font>PreTranslateMessage<font color="#FF0000">(</font>MSG<font color="#FF0000">*</font> pMsg<font color="#FF0000">)</font>
<font color="#FF0000">{</font>
    ... 
    <font color="#006600">//Catch requests to start another instance and restore window </font>
    <font color="#0088FF">if</font><font color="#FF0000">(</font>pMsg<font color="#FF0000">-</font><font color="#FF0000">></font><font color="#0088FF">message</font> <font color="#FF0000">=</font><font color="#FF0000">=</font> m_OtherAppMsgId <font color="#FF0000">)</font> 
    <font color="#FF0000">{</font> 
        <font color="#006600">// Bring the main window or its popup to</font>
        <font color="#006600">// the foreground</font>
        ASSERT<font color="#FF0000">(</font> m_pMainWnd<font color="#FF0000">)</font><font color="#FF0000">;</font> 
        <font color="#0088FF">if</font><font color="#FF0000">(</font> m_pMainWnd<font color="#FF0000">-</font><font color="#FF0000">></font>IsIconic<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">)</font>
            m_pMainWnd<font color="#FF0000">-</font><font color="#FF0000">></font>ShowWindow<font color="#FF0000">(</font>SW_RESTORE<font color="#FF0000">)</font><font color="#FF0000">;</font>
        m_pMainWnd<font color="#FF0000">-</font><font color="#FF0000">></font>SetForegroundWindow<font color="#FF0000">(</font><font color="#FF0000">)</font><font color="#FF0000">;</font>
    <font color="#FF0000">}</font>
    ...
    <font color="#0088FF">return</font> CWinApp<font color="#FF0000">:</font><font color="#FF0000">:</font>PreTranslateMessage<font color="#FF0000">(</font>pMsg<font color="#FF0000">)</font><font color="#FF0000">;</font>
<font color="#FF0000">}</font>
</pre>		
	
	
<!-- #include virtual="/Inc/NoteFooter.inc" --> 





